# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe._reversal import Reversal
    from stripe.params._transfer_reversal_create_params import (
        TransferReversalCreateParams,
    )
    from stripe.params._transfer_reversal_list_params import (
        TransferReversalListParams,
    )
    from stripe.params._transfer_reversal_retrieve_params import (
        TransferReversalRetrieveParams,
    )
    from stripe.params._transfer_reversal_update_params import (
        TransferReversalUpdateParams,
    )


class TransferReversalService(StripeService):
    def list(
        self,
        id: str,
        params: Optional["TransferReversalListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Reversal]":
        """
        You can see a list of the reversals belonging to a specific transfer. Note that the 10 most recent reversals are always available by default on the transfer object. If you need more than those 10, you can use this API method and the limit and starting_after parameters to page through additional reversals.
        """
        return cast(
            "ListObject[Reversal]",
            self._request(
                "get",
                "/v1/transfers/{id}/reversals".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        id: str,
        params: Optional["TransferReversalListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Reversal]":
        """
        You can see a list of the reversals belonging to a specific transfer. Note that the 10 most recent reversals are always available by default on the transfer object. If you need more than those 10, you can use this API method and the limit and starting_after parameters to page through additional reversals.
        """
        return cast(
            "ListObject[Reversal]",
            await self._request_async(
                "get",
                "/v1/transfers/{id}/reversals".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        id: str,
        params: Optional["TransferReversalCreateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Reversal":
        """
        When you create a new reversal, you must specify a transfer to create it on.

        When reversing transfers, you can optionally reverse part of the transfer. You can do so as many times as you wish until the entire transfer has been reversed.

        Once entirely reversed, a transfer can't be reversed again. This method will return an error when called on an already-reversed transfer, or when trying to reverse more money than is left on a transfer.
        """
        return cast(
            "Reversal",
            self._request(
                "post",
                "/v1/transfers/{id}/reversals".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        id: str,
        params: Optional["TransferReversalCreateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Reversal":
        """
        When you create a new reversal, you must specify a transfer to create it on.

        When reversing transfers, you can optionally reverse part of the transfer. You can do so as many times as you wish until the entire transfer has been reversed.

        Once entirely reversed, a transfer can't be reversed again. This method will return an error when called on an already-reversed transfer, or when trying to reverse more money than is left on a transfer.
        """
        return cast(
            "Reversal",
            await self._request_async(
                "post",
                "/v1/transfers/{id}/reversals".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        transfer: str,
        id: str,
        params: Optional["TransferReversalRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Reversal":
        """
        By default, you can see the 10 most recent reversals stored directly on the transfer object, but you can also retrieve details about a specific reversal stored on the transfer.
        """
        return cast(
            "Reversal",
            self._request(
                "get",
                "/v1/transfers/{transfer}/reversals/{id}".format(
                    transfer=sanitize_id(transfer),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        transfer: str,
        id: str,
        params: Optional["TransferReversalRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Reversal":
        """
        By default, you can see the 10 most recent reversals stored directly on the transfer object, but you can also retrieve details about a specific reversal stored on the transfer.
        """
        return cast(
            "Reversal",
            await self._request_async(
                "get",
                "/v1/transfers/{transfer}/reversals/{id}".format(
                    transfer=sanitize_id(transfer),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        transfer: str,
        id: str,
        params: Optional["TransferReversalUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Reversal":
        """
        Updates the specified reversal by setting the values of the parameters passed. Any parameters not provided will be left unchanged.

        This request only accepts metadata and description as arguments.
        """
        return cast(
            "Reversal",
            self._request(
                "post",
                "/v1/transfers/{transfer}/reversals/{id}".format(
                    transfer=sanitize_id(transfer),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        transfer: str,
        id: str,
        params: Optional["TransferReversalUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Reversal":
        """
        Updates the specified reversal by setting the values of the parameters passed. Any parameters not provided will be left unchanged.

        This request only accepts metadata and description as arguments.
        """
        return cast(
            "Reversal",
            await self._request_async(
                "post",
                "/v1/transfers/{transfer}/reversals/{id}".format(
                    transfer=sanitize_id(transfer),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.test_helpers.terminal._reader_present_payment_method_params import (
        ReaderPresentPaymentMethodParams,
    )
    from stripe.params.test_helpers.terminal._reader_succeed_input_collection_params import (
        ReaderSucceedInputCollectionParams,
    )
    from stripe.params.test_helpers.terminal._reader_timeout_input_collection_params import (
        ReaderTimeoutInputCollectionParams,
    )
    from stripe.terminal._reader import Reader


class ReaderService(StripeService):
    def present_payment_method(
        self,
        reader: str,
        params: Optional["ReaderPresentPaymentMethodParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Reader":
        """
        Presents a payment method on a simulated reader. Can be used to simulate accepting a payment, saving a card or refunding a transaction.
        """
        return cast(
            "Reader",
            self._request(
                "post",
                "/v1/test_helpers/terminal/readers/{reader}/present_payment_method".format(
                    reader=sanitize_id(reader),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def present_payment_method_async(
        self,
        reader: str,
        params: Optional["ReaderPresentPaymentMethodParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Reader":
        """
        Presents a payment method on a simulated reader. Can be used to simulate accepting a payment, saving a card or refunding a transaction.
        """
        return cast(
            "Reader",
            await self._request_async(
                "post",
                "/v1/test_helpers/terminal/readers/{reader}/present_payment_method".format(
                    reader=sanitize_id(reader),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def succeed_input_collection(
        self,
        reader: str,
        params: Optional["ReaderSucceedInputCollectionParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Reader":
        """
        Use this endpoint to trigger a successful input collection on a simulated reader.
        """
        return cast(
            "Reader",
            self._request(
                "post",
                "/v1/test_helpers/terminal/readers/{reader}/succeed_input_collection".format(
                    reader=sanitize_id(reader),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def succeed_input_collection_async(
        self,
        reader: str,
        params: Optional["ReaderSucceedInputCollectionParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Reader":
        """
        Use this endpoint to trigger a successful input collection on a simulated reader.
        """
        return cast(
            "Reader",
            await self._request_async(
                "post",
                "/v1/test_helpers/terminal/readers/{reader}/succeed_input_collection".format(
                    reader=sanitize_id(reader),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def timeout_input_collection(
        self,
        reader: str,
        params: Optional["ReaderTimeoutInputCollectionParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Reader":
        """
        Use this endpoint to complete an input collection with a timeout error on a simulated reader.
        """
        return cast(
            "Reader",
            self._request(
                "post",
                "/v1/test_helpers/terminal/readers/{reader}/timeout_input_collection".format(
                    reader=sanitize_id(reader),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def timeout_input_collection_async(
        self,
        reader: str,
        params: Optional["ReaderTimeoutInputCollectionParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Reader":
        """
        Use this endpoint to complete an input collection with a timeout error on a simulated reader.
        """
        return cast(
            "Reader",
            await self._request_async(
                "post",
                "/v1/test_helpers/terminal/readers/{reader}/timeout_input_collection".format(
                    reader=sanitize_id(reader),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.treasury._financial_account_features_retrieve_params import (
        FinancialAccountFeaturesRetrieveParams,
    )
    from stripe.params.treasury._financial_account_features_update_params import (
        FinancialAccountFeaturesUpdateParams,
    )
    from stripe.treasury._financial_account_features import (
        FinancialAccountFeatures,
    )


class FinancialAccountFeaturesService(StripeService):
    def update(
        self,
        financial_account: str,
        params: Optional["FinancialAccountFeaturesUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "FinancialAccountFeatures":
        """
        Updates the Features associated with a FinancialAccount.
        """
        return cast(
            "FinancialAccountFeatures",
            self._request(
                "post",
                "/v1/treasury/financial_accounts/{financial_account}/features".format(
                    financial_account=sanitize_id(financial_account),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        financial_account: str,
        params: Optional["FinancialAccountFeaturesUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "FinancialAccountFeatures":
        """
        Updates the Features associated with a FinancialAccount.
        """
        return cast(
            "FinancialAccountFeatures",
            await self._request_async(
                "post",
                "/v1/treasury/financial_accounts/{financial_account}/features".format(
                    financial_account=sanitize_id(financial_account),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        financial_account: str,
        params: Optional["FinancialAccountFeaturesRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "FinancialAccountFeatures":
        """
        Retrieves Features information associated with the FinancialAccount.
        """
        return cast(
            "FinancialAccountFeatures",
            self._request(
                "get",
                "/v1/treasury/financial_accounts/{financial_account}/features".format(
                    financial_account=sanitize_id(financial_account),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        financial_account: str,
        params: Optional["FinancialAccountFeaturesRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "FinancialAccountFeatures":
        """
        Retrieves Features information associated with the FinancialAccount.
        """
        return cast(
            "FinancialAccountFeatures",
            await self._request_async(
                "get",
                "/v1/treasury/financial_accounts/{financial_account}/features".format(
                    financial_account=sanitize_id(financial_account),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe._tax_rate import TaxRate
    from stripe.params._tax_rate_create_params import TaxRateCreateParams
    from stripe.params._tax_rate_list_params import TaxRateListParams
    from stripe.params._tax_rate_retrieve_params import TaxRateRetrieveParams
    from stripe.params._tax_rate_update_params import TaxRateUpdateParams


class TaxRateService(StripeService):
    def list(
        self,
        params: Optional["TaxRateListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[TaxRate]":
        """
        Returns a list of your tax rates. Tax rates are returned sorted by creation date, with the most recently created tax rates appearing first.
        """
        return cast(
            "ListObject[TaxRate]",
            self._request(
                "get",
                "/v1/tax_rates",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["TaxRateListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[TaxRate]":
        """
        Returns a list of your tax rates. Tax rates are returned sorted by creation date, with the most recently created tax rates appearing first.
        """
        return cast(
            "ListObject[TaxRate]",
            await self._request_async(
                "get",
                "/v1/tax_rates",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "TaxRateCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "TaxRate":
        """
        Creates a new tax rate.
        """
        return cast(
            "TaxRate",
            self._request(
                "post",
                "/v1/tax_rates",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "TaxRateCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "TaxRate":
        """
        Creates a new tax rate.
        """
        return cast(
            "TaxRate",
            await self._request_async(
                "post",
                "/v1/tax_rates",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        tax_rate: str,
        params: Optional["TaxRateRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "TaxRate":
        """
        Retrieves a tax rate with the given ID
        """
        return cast(
            "TaxRate",
            self._request(
                "get",
                "/v1/tax_rates/{tax_rate}".format(
                    tax_rate=sanitize_id(tax_rate),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        tax_rate: str,
        params: Optional["TaxRateRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "TaxRate":
        """
        Retrieves a tax rate with the given ID
        """
        return cast(
            "TaxRate",
            await self._request_async(
                "get",
                "/v1/tax_rates/{tax_rate}".format(
                    tax_rate=sanitize_id(tax_rate),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        tax_rate: str,
        params: Optional["TaxRateUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "TaxRate":
        """
        Updates an existing tax rate.
        """
        return cast(
            "TaxRate",
            self._request(
                "post",
                "/v1/tax_rates/{tax_rate}".format(
                    tax_rate=sanitize_id(tax_rate),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        tax_rate: str,
        params: Optional["TaxRateUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "TaxRate":
        """
        Updates an existing tax rate.
        """
        return cast(
            "TaxRate",
            await self._request_async(
                "post",
                "/v1/tax_rates/{tax_rate}".format(
                    tax_rate=sanitize_id(tax_rate),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import Dict, List
from typing_extensions import Literal, NotRequired, TypedDict


class InvoiceModifyParams(RequestOptions):
    account_tax_ids: NotRequired["Literal['']|List[str]"]
    """
    The account tax IDs associated with the invoice. Only editable when the invoice is a draft.
    """
    application_fee_amount: NotRequired[int]
    """
    A fee in cents (or local equivalent) that will be applied to the invoice and transferred to the application owner's Stripe account. The request must be made with an OAuth key or the Stripe-Account header in order to take an application fee. For more information, see the application fees [documentation](https://stripe.com/docs/billing/invoices/connect#collecting-fees).
    """
    auto_advance: NotRequired[bool]
    """
    Controls whether Stripe performs [automatic collection](https://stripe.com/docs/invoicing/integration/automatic-advancement-collection) of the invoice.
    """
    automatic_tax: NotRequired["InvoiceModifyParamsAutomaticTax"]
    """
    Settings for automatic tax lookup for this invoice.
    """
    automatically_finalizes_at: NotRequired[int]
    """
    The time when this invoice should be scheduled to finalize (up to 5 years in the future). The invoice is finalized at this time if it's still in draft state. To turn off automatic finalization, set `auto_advance` to false.
    """
    collection_method: NotRequired[
        Literal["charge_automatically", "send_invoice"]
    ]
    """
    Either `charge_automatically` or `send_invoice`. This field can be updated only on `draft` invoices.
    """
    custom_fields: NotRequired[
        "Literal['']|List[InvoiceModifyParamsCustomField]"
    ]
    """
    A list of up to 4 custom fields to be displayed on the invoice. If a value for `custom_fields` is specified, the list specified will replace the existing custom field list on this invoice. Pass an empty string to remove previously-defined fields.
    """
    days_until_due: NotRequired[int]
    """
    The number of days from which the invoice is created until it is due. Only valid for invoices where `collection_method=send_invoice`. This field can only be updated on `draft` invoices.
    """
    default_payment_method: NotRequired[str]
    """
    ID of the default payment method for the invoice. It must belong to the customer associated with the invoice. If not set, defaults to the subscription's default payment method, if any, or to the default payment method in the customer's invoice settings.
    """
    default_source: NotRequired["Literal['']|str"]
    """
    ID of the default payment source for the invoice. It must belong to the customer associated with the invoice and be in a chargeable state. If not set, defaults to the subscription's default source, if any, or to the customer's default source.
    """
    default_tax_rates: NotRequired["Literal['']|List[str]"]
    """
    The tax rates that will apply to any line item that does not have `tax_rates` set. Pass an empty string to remove previously-defined tax rates.
    """
    description: NotRequired[str]
    """
    An arbitrary string attached to the object. Often useful for displaying to users. Referenced as 'memo' in the Dashboard.
    """
    discounts: NotRequired["Literal['']|List[InvoiceModifyParamsDiscount]"]
    """
    The discounts that will apply to the invoice. Pass an empty string to remove previously-defined discounts.
    """
    due_date: NotRequired[int]
    """
    The date on which payment for this invoice is due. Only valid for invoices where `collection_method=send_invoice`. This field can only be updated on `draft` invoices.
    """
    effective_at: NotRequired["Literal['']|int"]
    """
    The date when this invoice is in effect. Same as `finalized_at` unless overwritten. When defined, this value replaces the system-generated 'Date of issue' printed on the invoice PDF and receipt.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    footer: NotRequired[str]
    """
    Footer to be displayed on the invoice.
    """
    issuer: NotRequired["InvoiceModifyParamsIssuer"]
    """
    The connected account that issues the invoice. The invoice is presented with the branding and support information of the specified account.
    """
    metadata: NotRequired["Literal['']|Dict[str, str]"]
    """
    Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    number: NotRequired["Literal['']|str"]
    """
    Set the number for this invoice. If no number is present then a number will be assigned automatically when the invoice is finalized. In many markets, regulations require invoices to be unique, sequential and / or gapless. You are responsible for ensuring this is true across all your different invoicing systems in the event that you edit the invoice number using our API. If you use only Stripe for your invoices and do not change invoice numbers, Stripe handles this aspect of compliance for you automatically.
    """
    on_behalf_of: NotRequired["Literal['']|str"]
    """
    The account (if any) for which the funds of the invoice payment are intended. If set, the invoice will be presented with the branding and support information of the specified account. See the [Invoices with Connect](https://stripe.com/docs/billing/invoices/connect) documentation for details.
    """
    payment_settings: NotRequired["InvoiceModifyParamsPaymentSettings"]
    """
    Configuration settings for the PaymentIntent that is generated when the invoice is finalized.
    """
    rendering: NotRequired["InvoiceModifyParamsRendering"]
    """
    The rendering-related settings that control how the invoice is displayed on customer-facing surfaces such as PDF and Hosted Invoice Page.
    """
    shipping_cost: NotRequired["Literal['']|InvoiceModifyParamsShippingCost"]
    """
    Settings for the cost of shipping for this invoice.
    """
    shipping_details: NotRequired[
        "Literal['']|InvoiceModifyParamsShippingDetails"
    ]
    """
    Shipping details for the invoice. The Invoice PDF will use the `shipping_details` value if it is set, otherwise the PDF will render the shipping address from the customer.
    """
    statement_descriptor: NotRequired[str]
    """
    Extra information about a charge for the customer's credit card statement. It must contain at least one letter. If not specified and this invoice is part of a subscription, the default `statement_descriptor` will be set to the first subscription item's product's `statement_descriptor`.
    """
    transfer_data: NotRequired["Literal['']|InvoiceModifyParamsTransferData"]
    """
    If specified, the funds from the invoice will be transferred to the destination and the ID of the resulting transfer will be found on the invoice's charge. This will be unset if you POST an empty value.
    """


class InvoiceModifyParamsAutomaticTax(TypedDict):
    enabled: bool
    """
    Whether Stripe automatically computes tax on this invoice. Note that incompatible invoice items (invoice items with manually specified [tax rates](https://stripe.com/docs/api/tax_rates), negative amounts, or `tax_behavior=unspecified`) cannot be added to automatic tax invoices.
    """
    liability: NotRequired["InvoiceModifyParamsAutomaticTaxLiability"]
    """
    The account that's liable for tax. If set, the business address and tax registrations required to perform the tax calculation are loaded from this account. The tax transaction is returned in the report of the connected account.
    """


class InvoiceModifyParamsAutomaticTaxLiability(TypedDict):
    account: NotRequired[str]
    """
    The connected account being referenced when `type` is `account`.
    """
    type: Literal["account", "self"]
    """
    Type of the account referenced in the request.
    """


class InvoiceModifyParamsCustomField(TypedDict):
    name: str
    """
    The name of the custom field. This may be up to 40 characters.
    """
    value: str
    """
    The value of the custom field. This may be up to 140 characters.
    """


class InvoiceModifyParamsDiscount(TypedDict):
    coupon: NotRequired[str]
    """
    ID of the coupon to create a new discount for.
    """
    discount: NotRequired[str]
    """
    ID of an existing discount on the object (or one of its ancestors) to reuse.
    """
    promotion_code: NotRequired[str]
    """
    ID of the promotion code to create a new discount for.
    """


class InvoiceModifyParamsIssuer(TypedDict):
    account: NotRequired[str]
    """
    The connected account being referenced when `type` is `account`.
    """
    type: Literal["account", "self"]
    """
    Type of the account referenced in the request.
    """


class InvoiceModifyParamsPaymentSettings(TypedDict):
    default_mandate: NotRequired["Literal['']|str"]
    """
    ID of the mandate to be used for this invoice. It must correspond to the payment method used to pay the invoice, including the invoice's default_payment_method or default_source, if set.
    """
    payment_method_options: NotRequired[
        "InvoiceModifyParamsPaymentSettingsPaymentMethodOptions"
    ]
    """
    Payment-method-specific configuration to provide to the invoice's PaymentIntent.
    """
    payment_method_types: NotRequired[
        "Literal['']|List[Literal['ach_credit_transfer', 'ach_debit', 'acss_debit', 'affirm', 'amazon_pay', 'au_becs_debit', 'bacs_debit', 'bancontact', 'boleto', 'card', 'cashapp', 'crypto', 'custom', 'customer_balance', 'eps', 'fpx', 'giropay', 'grabpay', 'ideal', 'jp_credit_transfer', 'kakao_pay', 'klarna', 'konbini', 'kr_card', 'link', 'multibanco', 'naver_pay', 'nz_bank_account', 'p24', 'payco', 'paynow', 'paypal', 'promptpay', 'revolut_pay', 'sepa_credit_transfer', 'sepa_debit', 'sofort', 'swish', 'us_bank_account', 'wechat_pay']]"
    ]
    """
    The list of payment method types (e.g. card) to provide to the invoice's PaymentIntent. If not set, Stripe attempts to automatically determine the types to use by looking at the invoice's default payment method, the subscription's default payment method, the customer's default payment method, and your [invoice template settings](https://dashboard.stripe.com/settings/billing/invoice).
    """


class InvoiceModifyParamsPaymentSettingsPaymentMethodOptions(TypedDict):
    acss_debit: NotRequired[
        "Literal['']|InvoiceModifyParamsPaymentSettingsPaymentMethodOptionsAcssDebit"
    ]
    """
    If paying by `acss_debit`, this sub-hash contains details about the Canadian pre-authorized debit payment method options to pass to the invoice's PaymentIntent.
    """
    bancontact: NotRequired[
        "Literal['']|InvoiceModifyParamsPaymentSettingsPaymentMethodOptionsBancontact"
    ]
    """
    If paying by `bancontact`, this sub-hash contains details about the Bancontact payment method options to pass to the invoice's PaymentIntent.
    """
    card: NotRequired[
        "Literal['']|InvoiceModifyParamsPaymentSettingsPaymentMethodOptionsCard"
    ]
    """
    If paying by `card`, this sub-hash contains details about the Card payment method options to pass to the invoice's PaymentIntent.
    """
    customer_balance: NotRequired[
        "Literal['']|InvoiceModifyParamsPaymentSettingsPaymentMethodOptionsCustomerBalance"
    ]
    """
    If paying by `customer_balance`, this sub-hash contains details about the Bank transfer payment method options to pass to the invoice's PaymentIntent.
    """
    konbini: NotRequired[
        "Literal['']|InvoiceModifyParamsPaymentSettingsPaymentMethodOptionsKonbini"
    ]
    """
    If paying by `konbini`, this sub-hash contains details about the Konbini payment method options to pass to the invoice's PaymentIntent.
    """
    sepa_debit: NotRequired[
        "Literal['']|InvoiceModifyParamsPaymentSettingsPaymentMethodOptionsSepaDebit"
    ]
    """
    If paying by `sepa_debit`, this sub-hash contains details about the SEPA Direct Debit payment method options to pass to the invoice's PaymentIntent.
    """
    us_bank_account: NotRequired[
        "Literal['']|InvoiceModifyParamsPaymentSettingsPaymentMethodOptionsUsBankAccount"
    ]
    """
    If paying by `us_bank_account`, this sub-hash contains details about the ACH direct debit payment method options to pass to the invoice's PaymentIntent.
    """


class InvoiceModifyParamsPaymentSettingsPaymentMethodOptionsAcssDebit(
    TypedDict,
):
    mandate_options: NotRequired[
        "InvoiceModifyParamsPaymentSettingsPaymentMethodOptionsAcssDebitMandateOptions"
    ]
    """
    Additional fields for Mandate creation
    """
    verification_method: NotRequired[
        Literal["automatic", "instant", "microdeposits"]
    ]
    """
    Verification method for the intent
    """


class InvoiceModifyParamsPaymentSettingsPaymentMethodOptionsAcssDebitMandateOptions(
    TypedDict,
):
    transaction_type: NotRequired[Literal["business", "personal"]]
    """
    Transaction type of the mandate.
    """


class InvoiceModifyParamsPaymentSettingsPaymentMethodOptionsBancontact(
    TypedDict,
):
    preferred_language: NotRequired[Literal["de", "en", "fr", "nl"]]
    """
    Preferred language of the Bancontact authorization page that the customer is redirected to.
    """


class InvoiceModifyParamsPaymentSettingsPaymentMethodOptionsCard(TypedDict):
    installments: NotRequired[
        "InvoiceModifyParamsPaymentSettingsPaymentMethodOptionsCardInstallments"
    ]
    """
    Installment configuration for payments attempted on this invoice.

    For more information, see the [installments integration guide](https://stripe.com/docs/payments/installments).
    """
    request_three_d_secure: NotRequired[
        Literal["any", "automatic", "challenge"]
    ]
    """
    We strongly recommend that you rely on our SCA Engine to automatically prompt your customers for authentication based on risk level and [other requirements](https://stripe.com/docs/strong-customer-authentication). However, if you wish to request 3D Secure based on logic from your own fraud engine, provide this option. Read our guide on [manually requesting 3D Secure](https://stripe.com/docs/payments/3d-secure/authentication-flow#manual-three-ds) for more information on how this configuration interacts with Radar and our SCA Engine.
    """


class InvoiceModifyParamsPaymentSettingsPaymentMethodOptionsCardInstallments(
    TypedDict,
):
    enabled: NotRequired[bool]
    """
    Setting to true enables installments for this invoice.
    Setting to false will prevent any selected plan from applying to a payment.
    """
    plan: NotRequired[
        "Literal['']|InvoiceModifyParamsPaymentSettingsPaymentMethodOptionsCardInstallmentsPlan"
    ]
    """
    The selected installment plan to use for this invoice.
    """


class InvoiceModifyParamsPaymentSettingsPaymentMethodOptionsCardInstallmentsPlan(
    TypedDict,
):
    count: NotRequired[int]
    """
    For `fixed_count` installment plans, this is required. It represents the number of installment payments your customer will make to their credit card.
    """
    interval: NotRequired[Literal["month"]]
    """
    For `fixed_count` installment plans, this is required. It represents the interval between installment payments your customer will make to their credit card.
    One of `month`.
    """
    type: Literal["bonus", "fixed_count", "revolving"]
    """
    Type of installment plan, one of `fixed_count`, `bonus`, or `revolving`.
    """


class InvoiceModifyParamsPaymentSettingsPaymentMethodOptionsCustomerBalance(
    TypedDict,
):
    bank_transfer: NotRequired[
        "InvoiceModifyParamsPaymentSettingsPaymentMethodOptionsCustomerBalanceBankTransfer"
    ]
    """
    Configuration for the bank transfer funding type, if the `funding_type` is set to `bank_transfer`.
    """
    funding_type: NotRequired[str]
    """
    The funding method type to be used when there are not enough funds in the customer balance. Permitted values include: `bank_transfer`.
    """


class InvoiceModifyParamsPaymentSettingsPaymentMethodOptionsCustomerBalanceBankTransfer(
    TypedDict,
):
    eu_bank_transfer: NotRequired[
        "InvoiceModifyParamsPaymentSettingsPaymentMethodOptionsCustomerBalanceBankTransferEuBankTransfer"
    ]
    """
    Configuration for eu_bank_transfer funding type.
    """
    type: NotRequired[str]
    """
    The bank transfer type that can be used for funding. Permitted values include: `eu_bank_transfer`, `gb_bank_transfer`, `jp_bank_transfer`, `mx_bank_transfer`, or `us_bank_transfer`.
    """


class InvoiceModifyParamsPaymentSettingsPaymentMethodOptionsCustomerBalanceBankTransferEuBankTransfer(
    TypedDict,
):
    country: str
    """
    The desired country code of the bank account information. Permitted values include: `BE`, `DE`, `ES`, `FR`, `IE`, or `NL`.
    """


class InvoiceModifyParamsPaymentSettingsPaymentMethodOptionsKonbini(TypedDict):
    pass


class InvoiceModifyParamsPaymentSettingsPaymentMethodOptionsSepaDebit(
    TypedDict,
):
    pass


class InvoiceModifyParamsPaymentSettingsPaymentMethodOptionsUsBankAccount(
    TypedDict,
):
    financial_connections: NotRequired[
        "InvoiceModifyParamsPaymentSettingsPaymentMethodOptionsUsBankAccountFinancialConnections"
    ]
    """
    Additional fields for Financial Connections Session creation
    """
    verification_method: NotRequired[
        Literal["automatic", "instant", "microdeposits"]
    ]
    """
    Verification method for the intent
    """


class InvoiceModifyParamsPaymentSettingsPaymentMethodOptionsUsBankAccountFinancialConnections(
    TypedDict,
):
    filters: NotRequired[
        "InvoiceModifyParamsPaymentSettingsPaymentMethodOptionsUsBankAccountFinancialConnectionsFilters"
    ]
    """
    Provide filters for the linked accounts that the customer can select for the payment method.
    """
    permissions: NotRequired[
        List[
            Literal["balances", "ownership", "payment_method", "transactions"]
        ]
    ]
    """
    The list of permissions to request. If this parameter is passed, the `payment_method` permission must be included. Valid permissions include: `balances`, `ownership`, `payment_method`, and `transactions`.
    """
    prefetch: NotRequired[
        List[Literal["balances", "ownership", "transactions"]]
    ]
    """
    List of data features that you would like to retrieve upon account creation.
    """


class InvoiceModifyParamsPaymentSettingsPaymentMethodOptionsUsBankAccountFinancialConnectionsFilters(
    TypedDict,
):
    account_subcategories: NotRequired[List[Literal["checking", "savings"]]]
    """
    The account subcategories to use to filter for selectable accounts. Valid subcategories are `checking` and `savings`.
    """


class InvoiceModifyParamsRendering(TypedDict):
    amount_tax_display: NotRequired[
        "Literal['']|Literal['exclude_tax', 'include_inclusive_tax']"
    ]
    """
    How line-item prices and amounts will be displayed with respect to tax on invoice PDFs. One of `exclude_tax` or `include_inclusive_tax`. `include_inclusive_tax` will include inclusive tax (and exclude exclusive tax) in invoice PDF amounts. `exclude_tax` will exclude all tax (inclusive and exclusive alike) from invoice PDF amounts.
    """
    pdf: NotRequired["InvoiceModifyParamsRenderingPdf"]
    """
    Invoice pdf rendering options
    """
    template: NotRequired[str]
    """
    ID of the invoice rendering template to use for this invoice.
    """
    template_version: NotRequired["Literal['']|int"]
    """
    The specific version of invoice rendering template to use for this invoice.
    """


class InvoiceModifyParamsRenderingPdf(TypedDict):
    page_size: NotRequired[Literal["a4", "auto", "letter"]]
    """
    Page size for invoice PDF. Can be set to `a4`, `letter`, or `auto`.
     If set to `auto`, invoice PDF page size defaults to `a4` for customers with
     Japanese locale and `letter` for customers with other locales.
    """


class InvoiceModifyParamsShippingCost(TypedDict):
    shipping_rate: NotRequired[str]
    """
    The ID of the shipping rate to use for this order.
    """
    shipping_rate_data: NotRequired[
        "InvoiceModifyParamsShippingCostShippingRateData"
    ]
    """
    Parameters to create a new ad-hoc shipping rate for this order.
    """


class InvoiceModifyParamsShippingCostShippingRateData(TypedDict):
    delivery_estimate: NotRequired[
        "InvoiceModifyParamsShippingCostShippingRateDataDeliveryEstimate"
    ]
    """
    The estimated range for how long shipping will take, meant to be displayable to the customer. This will appear on CheckoutSessions.
    """
    display_name: str
    """
    The name of the shipping rate, meant to be displayable to the customer. This will appear on CheckoutSessions.
    """
    fixed_amount: NotRequired[
        "InvoiceModifyParamsShippingCostShippingRateDataFixedAmount"
    ]
    """
    Describes a fixed amount to charge for shipping. Must be present if type is `fixed_amount`.
    """
    metadata: NotRequired[Dict[str, str]]
    """
    Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    tax_behavior: NotRequired[Literal["exclusive", "inclusive", "unspecified"]]
    """
    Specifies whether the rate is considered inclusive of taxes or exclusive of taxes. One of `inclusive`, `exclusive`, or `unspecified`.
    """
    tax_code: NotRequired[str]
    """
    A [tax code](https://stripe.com/docs/tax/tax-categories) ID. The Shipping tax code is `txcd_92010001`.
    """
    type: NotRequired[Literal["fixed_amount"]]
    """
    The type of calculation to use on the shipping rate.
    """


class InvoiceModifyParamsShippingCostShippingRateDataDeliveryEstimate(
    TypedDict,
):
    maximum: NotRequired[
        "InvoiceModifyParamsShippingCostShippingRateDataDeliveryEstimateMaximum"
    ]
    """
    The upper bound of the estimated range. If empty, represents no upper bound i.e., infinite.
    """
    minimum: NotRequired[
        "InvoiceModifyParamsShippingCostShippingRateDataDeliveryEstimateMinimum"
    ]
    """
    The lower bound of the estimated range. If empty, represents no lower bound.
    """


class InvoiceModifyParamsShippingCostShippingRateDataDeliveryEstimateMaximum(
    TypedDict,
):
    unit: Literal["business_day", "day", "hour", "month", "week"]
    """
    A unit of time.
    """
    value: int
    """
    Must be greater than 0.
    """


class InvoiceModifyParamsShippingCostShippingRateDataDeliveryEstimateMinimum(
    TypedDict,
):
    unit: Literal["business_day", "day", "hour", "month", "week"]
    """
    A unit of time.
    """
    value: int
    """
    Must be greater than 0.
    """


class InvoiceModifyParamsShippingCostShippingRateDataFixedAmount(TypedDict):
    amount: int
    """
    A non-negative integer in cents representing how much to charge.
    """
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    currency_options: NotRequired[
        Dict[
            str,
            "InvoiceModifyParamsShippingCostShippingRateDataFixedAmountCurrencyOptions",
        ]
    ]
    """
    Shipping rates defined in each available currency option. Each key must be a three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html) and a [supported currency](https://stripe.com/docs/currencies).
    """


class InvoiceModifyParamsShippingCostShippingRateDataFixedAmountCurrencyOptions(
    TypedDict,
):
    amount: int
    """
    A non-negative integer in cents representing how much to charge.
    """
    tax_behavior: NotRequired[Literal["exclusive", "inclusive", "unspecified"]]
    """
    Specifies whether the rate is considered inclusive of taxes or exclusive of taxes. One of `inclusive`, `exclusive`, or `unspecified`.
    """


class InvoiceModifyParamsShippingDetails(TypedDict):
    address: "InvoiceModifyParamsShippingDetailsAddress"
    """
    Shipping address
    """
    name: str
    """
    Recipient name.
    """
    phone: NotRequired["Literal['']|str"]
    """
    Recipient phone (including extension)
    """


class InvoiceModifyParamsShippingDetailsAddress(TypedDict):
    city: NotRequired[str]
    """
    City, district, suburb, town, or village.
    """
    country: NotRequired[str]
    """
    Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    line1: NotRequired[str]
    """
    Address line 1, such as the street, PO Box, or company name.
    """
    line2: NotRequired[str]
    """
    Address line 2, such as the apartment, suite, unit, or building.
    """
    postal_code: NotRequired[str]
    """
    ZIP or postal code.
    """
    state: NotRequired[str]
    """
    State, county, province, or region.
    """


class InvoiceModifyParamsTransferData(TypedDict):
    amount: NotRequired[int]
    """
    The amount that will be transferred automatically when the invoice is paid. If no amount is set, the full amount is transferred.
    """
    destination: str
    """
    ID of an existing, connected Stripe account.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params.v2.billing._meter_event_adjustment_create_params import (
        MeterEventAdjustmentCreateParams as MeterEventAdjustmentCreateParams,
        MeterEventAdjustmentCreateParamsCancel as MeterEventAdjustmentCreateParamsCancel,
    )
    from stripe.params.v2.billing._meter_event_create_params import (
        MeterEventCreateParams as MeterEventCreateParams,
    )
    from stripe.params.v2.billing._meter_event_session_create_params import (
        MeterEventSessionCreateParams as MeterEventSessionCreateParams,
    )
    from stripe.params.v2.billing._meter_event_stream_create_params import (
        MeterEventStreamCreateParams as MeterEventStreamCreateParams,
        MeterEventStreamCreateParamsEvent as MeterEventStreamCreateParamsEvent,
    )

# name -> (import_target, is_submodule)
_import_map = {
    "MeterEventAdjustmentCreateParams": (
        "stripe.params.v2.billing._meter_event_adjustment_create_params",
        False,
    ),
    "MeterEventAdjustmentCreateParamsCancel": (
        "stripe.params.v2.billing._meter_event_adjustment_create_params",
        False,
    ),
    "MeterEventCreateParams": (
        "stripe.params.v2.billing._meter_event_create_params",
        False,
    ),
    "MeterEventSessionCreateParams": (
        "stripe.params.v2.billing._meter_event_session_create_params",
        False,
    ),
    "MeterEventStreamCreateParams": (
        "stripe.params.v2.billing._meter_event_stream_create_params",
        False,
    ),
    "MeterEventStreamCreateParamsEvent": (
        "stripe.params.v2.billing._meter_event_stream_create_params",
        False,
    ),
}
if not TYPE_CHECKING:

    def __getattr__(name):
        try:
            target, is_submodule = _import_map[name]
            module = import_module(target)
            if is_submodule:
                return module

            return getattr(
                module,
                name,
            )
        except KeyError:
            raise AttributeError()



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe.issuing._cardholder import Cardholder
    from stripe.params.issuing._cardholder_create_params import (
        CardholderCreateParams,
    )
    from stripe.params.issuing._cardholder_list_params import (
        CardholderListParams,
    )
    from stripe.params.issuing._cardholder_retrieve_params import (
        CardholderRetrieveParams,
    )
    from stripe.params.issuing._cardholder_update_params import (
        CardholderUpdateParams,
    )


class CardholderService(StripeService):
    def list(
        self,
        params: Optional["CardholderListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Cardholder]":
        """
        Returns a list of Issuing Cardholder objects. The objects are sorted in descending order by creation date, with the most recently created object appearing first.
        """
        return cast(
            "ListObject[Cardholder]",
            self._request(
                "get",
                "/v1/issuing/cardholders",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["CardholderListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Cardholder]":
        """
        Returns a list of Issuing Cardholder objects. The objects are sorted in descending order by creation date, with the most recently created object appearing first.
        """
        return cast(
            "ListObject[Cardholder]",
            await self._request_async(
                "get",
                "/v1/issuing/cardholders",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "CardholderCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Cardholder":
        """
        Creates a new Issuing Cardholder object that can be issued cards.
        """
        return cast(
            "Cardholder",
            self._request(
                "post",
                "/v1/issuing/cardholders",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "CardholderCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Cardholder":
        """
        Creates a new Issuing Cardholder object that can be issued cards.
        """
        return cast(
            "Cardholder",
            await self._request_async(
                "post",
                "/v1/issuing/cardholders",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        cardholder: str,
        params: Optional["CardholderRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Cardholder":
        """
        Retrieves an Issuing Cardholder object.
        """
        return cast(
            "Cardholder",
            self._request(
                "get",
                "/v1/issuing/cardholders/{cardholder}".format(
                    cardholder=sanitize_id(cardholder),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        cardholder: str,
        params: Optional["CardholderRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Cardholder":
        """
        Retrieves an Issuing Cardholder object.
        """
        return cast(
            "Cardholder",
            await self._request_async(
                "get",
                "/v1/issuing/cardholders/{cardholder}".format(
                    cardholder=sanitize_id(cardholder),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        cardholder: str,
        params: Optional["CardholderUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Cardholder":
        """
        Updates the specified Issuing Cardholder object by setting the values of the parameters passed. Any parameters not provided will be left unchanged.
        """
        return cast(
            "Cardholder",
            self._request(
                "post",
                "/v1/issuing/cardholders/{cardholder}".format(
                    cardholder=sanitize_id(cardholder),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        cardholder: str,
        params: Optional["CardholderUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Cardholder":
        """
        Updates the specified Issuing Cardholder object by setting the values of the parameters passed. Any parameters not provided will be left unchanged.
        """
        return cast(
            "Cardholder",
            await self._request_async(
                "post",
                "/v1/issuing/cardholders/{cardholder}".format(
                    cardholder=sanitize_id(cardholder),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._apple_pay_domain import ApplePayDomain
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe.params._apple_pay_domain_create_params import (
        ApplePayDomainCreateParams,
    )
    from stripe.params._apple_pay_domain_delete_params import (
        ApplePayDomainDeleteParams,
    )
    from stripe.params._apple_pay_domain_list_params import (
        ApplePayDomainListParams,
    )
    from stripe.params._apple_pay_domain_retrieve_params import (
        ApplePayDomainRetrieveParams,
    )


class ApplePayDomainService(StripeService):
    def delete(
        self,
        domain: str,
        params: Optional["ApplePayDomainDeleteParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ApplePayDomain":
        """
        Delete an apple pay domain.
        """
        return cast(
            "ApplePayDomain",
            self._request(
                "delete",
                "/v1/apple_pay/domains/{domain}".format(
                    domain=sanitize_id(domain),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def delete_async(
        self,
        domain: str,
        params: Optional["ApplePayDomainDeleteParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ApplePayDomain":
        """
        Delete an apple pay domain.
        """
        return cast(
            "ApplePayDomain",
            await self._request_async(
                "delete",
                "/v1/apple_pay/domains/{domain}".format(
                    domain=sanitize_id(domain),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        domain: str,
        params: Optional["ApplePayDomainRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ApplePayDomain":
        """
        Retrieve an apple pay domain.
        """
        return cast(
            "ApplePayDomain",
            self._request(
                "get",
                "/v1/apple_pay/domains/{domain}".format(
                    domain=sanitize_id(domain),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        domain: str,
        params: Optional["ApplePayDomainRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ApplePayDomain":
        """
        Retrieve an apple pay domain.
        """
        return cast(
            "ApplePayDomain",
            await self._request_async(
                "get",
                "/v1/apple_pay/domains/{domain}".format(
                    domain=sanitize_id(domain),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def list(
        self,
        params: Optional["ApplePayDomainListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[ApplePayDomain]":
        """
        List apple pay domains.
        """
        return cast(
            "ListObject[ApplePayDomain]",
            self._request(
                "get",
                "/v1/apple_pay/domains",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["ApplePayDomainListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[ApplePayDomain]":
        """
        List apple pay domains.
        """
        return cast(
            "ListObject[ApplePayDomain]",
            await self._request_async(
                "get",
                "/v1/apple_pay/domains",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "ApplePayDomainCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "ApplePayDomain":
        """
        Create an apple pay domain.
        """
        return cast(
            "ApplePayDomain",
            self._request(
                "post",
                "/v1/apple_pay/domains",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "ApplePayDomainCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "ApplePayDomain":
        """
        Create an apple pay domain.
        """
        return cast(
            "ApplePayDomain",
            await self._request_async(
                "post",
                "/v1/apple_pay/domains",
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._account import Account
    from stripe._bank_account import BankAccount
    from stripe._card import Card
    from stripe._request_options import RequestOptions
    from stripe._source import Source
    from stripe._source_transaction_service import SourceTransactionService
    from stripe.params._source_create_params import SourceCreateParams
    from stripe.params._source_detach_params import SourceDetachParams
    from stripe.params._source_retrieve_params import SourceRetrieveParams
    from stripe.params._source_update_params import SourceUpdateParams
    from stripe.params._source_verify_params import SourceVerifyParams
    from typing import Union

_subservices = {
    "transactions": [
        "stripe._source_transaction_service",
        "SourceTransactionService",
    ],
}


class SourceService(StripeService):
    transactions: "SourceTransactionService"

    def __init__(self, requestor):
        super().__init__(requestor)

    def __getattr__(self, name):
        try:
            import_from, service = _subservices[name]
            service_class = getattr(
                import_module(import_from),
                service,
            )
            setattr(
                self,
                name,
                service_class(self._requestor),
            )
            return getattr(self, name)
        except KeyError:
            raise AttributeError()

    def detach(
        self,
        customer: str,
        id: str,
        params: Optional["SourceDetachParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Union[Account, BankAccount, Card, Source]":
        """
        Delete a specified source for a given customer.
        """
        return cast(
            "Union[Account, BankAccount, Card, Source]",
            self._request(
                "delete",
                "/v1/customers/{customer}/sources/{id}".format(
                    customer=sanitize_id(customer),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def detach_async(
        self,
        customer: str,
        id: str,
        params: Optional["SourceDetachParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Union[Account, BankAccount, Card, Source]":
        """
        Delete a specified source for a given customer.
        """
        return cast(
            "Union[Account, BankAccount, Card, Source]",
            await self._request_async(
                "delete",
                "/v1/customers/{customer}/sources/{id}".format(
                    customer=sanitize_id(customer),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        source: str,
        params: Optional["SourceRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Source":
        """
        Retrieves an existing source object. Supply the unique source ID from a source creation request and Stripe will return the corresponding up-to-date source object information.
        """
        return cast(
            "Source",
            self._request(
                "get",
                "/v1/sources/{source}".format(source=sanitize_id(source)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        source: str,
        params: Optional["SourceRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Source":
        """
        Retrieves an existing source object. Supply the unique source ID from a source creation request and Stripe will return the corresponding up-to-date source object information.
        """
        return cast(
            "Source",
            await self._request_async(
                "get",
                "/v1/sources/{source}".format(source=sanitize_id(source)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        source: str,
        params: Optional["SourceUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Source":
        """
        Updates the specified source by setting the values of the parameters passed. Any parameters not provided will be left unchanged.

        This request accepts the metadata and owner as arguments. It is also possible to update type specific information for selected payment methods. Please refer to our [payment method guides](https://docs.stripe.com/docs/sources) for more detail.
        """
        return cast(
            "Source",
            self._request(
                "post",
                "/v1/sources/{source}".format(source=sanitize_id(source)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        source: str,
        params: Optional["SourceUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Source":
        """
        Updates the specified source by setting the values of the parameters passed. Any parameters not provided will be left unchanged.

        This request accepts the metadata and owner as arguments. It is also possible to update type specific information for selected payment methods. Please refer to our [payment method guides](https://docs.stripe.com/docs/sources) for more detail.
        """
        return cast(
            "Source",
            await self._request_async(
                "post",
                "/v1/sources/{source}".format(source=sanitize_id(source)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: Optional["SourceCreateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Source":
        """
        Creates a new source object.
        """
        return cast(
            "Source",
            self._request(
                "post",
                "/v1/sources",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: Optional["SourceCreateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Source":
        """
        Creates a new source object.
        """
        return cast(
            "Source",
            await self._request_async(
                "post",
                "/v1/sources",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def verify(
        self,
        source: str,
        params: "SourceVerifyParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Source":
        """
        Verify a given source.
        """
        return cast(
            "Source",
            self._request(
                "post",
                "/v1/sources/{source}/verify".format(
                    source=sanitize_id(source),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def verify_async(
        self,
        source: str,
        params: "SourceVerifyParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Source":
        """
        Verify a given source.
        """
        return cast(
            "Source",
            await self._request_async(
                "post",
                "/v1/sources/{source}/verify".format(
                    source=sanitize_id(source),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._createable_api_resource import CreateableAPIResource
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._stripe_object import StripeObject
from stripe._updateable_api_resource import UpdateableAPIResource
from stripe._util import class_method_variant, sanitize_id
from typing import ClassVar, Dict, List, Optional, cast, overload
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params.treasury._financial_account_close_params import (
        FinancialAccountCloseParams,
    )
    from stripe.params.treasury._financial_account_create_params import (
        FinancialAccountCreateParams,
    )
    from stripe.params.treasury._financial_account_list_params import (
        FinancialAccountListParams,
    )
    from stripe.params.treasury._financial_account_modify_params import (
        FinancialAccountModifyParams,
    )
    from stripe.params.treasury._financial_account_retrieve_features_params import (
        FinancialAccountRetrieveFeaturesParams,
    )
    from stripe.params.treasury._financial_account_retrieve_params import (
        FinancialAccountRetrieveParams,
    )
    from stripe.params.treasury._financial_account_update_features_params import (
        FinancialAccountUpdateFeaturesParams,
    )
    from stripe.treasury._financial_account_features import (
        FinancialAccountFeatures,
    )


class FinancialAccount(
    CreateableAPIResource["FinancialAccount"],
    ListableAPIResource["FinancialAccount"],
    UpdateableAPIResource["FinancialAccount"],
):
    """
    Stripe Treasury provides users with a container for money called a FinancialAccount that is separate from their Payments balance.
    FinancialAccounts serve as the source and destination of Treasury's money movement APIs.
    """

    OBJECT_NAME: ClassVar[Literal["treasury.financial_account"]] = (
        "treasury.financial_account"
    )

    class Balance(StripeObject):
        cash: Dict[str, int]
        """
        Funds the user can spend right now.
        """
        inbound_pending: Dict[str, int]
        """
        Funds not spendable yet, but will become available at a later time.
        """
        outbound_pending: Dict[str, int]
        """
        Funds in the account, but not spendable because they are being held for pending outbound flows.
        """

    class FinancialAddress(StripeObject):
        class Aba(StripeObject):
            account_holder_name: str
            """
            The name of the person or business that owns the bank account.
            """
            account_number: Optional[str]
            """
            The account number.
            """
            account_number_last4: str
            """
            The last four characters of the account number.
            """
            bank_name: str
            """
            Name of the bank.
            """
            routing_number: str
            """
            Routing number for the account.
            """

        aba: Optional[Aba]
        """
        ABA Records contain U.S. bank account details per the ABA format.
        """
        supported_networks: Optional[List[Literal["ach", "us_domestic_wire"]]]
        """
        The list of networks that the address supports
        """
        type: Literal["aba"]
        """
        The type of financial address
        """
        _inner_class_types = {"aba": Aba}

    class PlatformRestrictions(StripeObject):
        inbound_flows: Optional[Literal["restricted", "unrestricted"]]
        """
        Restricts all inbound money movement.
        """
        outbound_flows: Optional[Literal["restricted", "unrestricted"]]
        """
        Restricts all outbound money movement.
        """

    class StatusDetails(StripeObject):
        class Closed(StripeObject):
            reasons: List[
                Literal["account_rejected", "closed_by_platform", "other"]
            ]
            """
            The array that contains reasons for a FinancialAccount closure.
            """

        closed: Optional[Closed]
        """
        Details related to the closure of this FinancialAccount
        """
        _inner_class_types = {"closed": Closed}

    active_features: Optional[
        List[
            Literal[
                "card_issuing",
                "deposit_insurance",
                "financial_addresses.aba",
                "financial_addresses.aba.forwarding",
                "inbound_transfers.ach",
                "intra_stripe_flows",
                "outbound_payments.ach",
                "outbound_payments.us_domestic_wire",
                "outbound_transfers.ach",
                "outbound_transfers.us_domestic_wire",
                "remote_deposit_capture",
            ]
        ]
    ]
    """
    The array of paths to active Features in the Features hash.
    """
    balance: Balance
    """
    Balance information for the FinancialAccount
    """
    country: str
    """
    Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    features: Optional["FinancialAccountFeatures"]
    """
    Encodes whether a FinancialAccount has access to a particular Feature, with a `status` enum and associated `status_details`.
    Stripe or the platform can control Features via the requested field.
    """
    financial_addresses: List[FinancialAddress]
    """
    The set of credentials that resolve to a FinancialAccount.
    """
    id: str
    """
    Unique identifier for the object.
    """
    is_default: Optional[bool]
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    metadata: Optional[Dict[str, str]]
    """
    Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    nickname: Optional[str]
    """
    The nickname for the FinancialAccount.
    """
    object: Literal["treasury.financial_account"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    pending_features: Optional[
        List[
            Literal[
                "card_issuing",
                "deposit_insurance",
                "financial_addresses.aba",
                "financial_addresses.aba.forwarding",
                "inbound_transfers.ach",
                "intra_stripe_flows",
                "outbound_payments.ach",
                "outbound_payments.us_domestic_wire",
                "outbound_transfers.ach",
                "outbound_transfers.us_domestic_wire",
                "remote_deposit_capture",
            ]
        ]
    ]
    """
    The array of paths to pending Features in the Features hash.
    """
    platform_restrictions: Optional[PlatformRestrictions]
    """
    The set of functionalities that the platform can restrict on the FinancialAccount.
    """
    restricted_features: Optional[
        List[
            Literal[
                "card_issuing",
                "deposit_insurance",
                "financial_addresses.aba",
                "financial_addresses.aba.forwarding",
                "inbound_transfers.ach",
                "intra_stripe_flows",
                "outbound_payments.ach",
                "outbound_payments.us_domestic_wire",
                "outbound_transfers.ach",
                "outbound_transfers.us_domestic_wire",
                "remote_deposit_capture",
            ]
        ]
    ]
    """
    The array of paths to restricted Features in the Features hash.
    """
    status: Literal["closed", "open"]
    """
    Status of this FinancialAccount.
    """
    status_details: StatusDetails
    supported_currencies: List[str]
    """
    The currencies the FinancialAccount can hold a balance in. Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase.
    """

    @classmethod
    def _cls_close(
        cls,
        financial_account: str,
        **params: Unpack["FinancialAccountCloseParams"],
    ) -> "FinancialAccount":
        """
        Closes a FinancialAccount. A FinancialAccount can only be closed if it has a zero balance, has no pending InboundTransfers, and has canceled all attached Issuing cards.
        """
        return cast(
            "FinancialAccount",
            cls._static_request(
                "post",
                "/v1/treasury/financial_accounts/{financial_account}/close".format(
                    financial_account=sanitize_id(financial_account)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    def close(
        financial_account: str, **params: Unpack["FinancialAccountCloseParams"]
    ) -> "FinancialAccount":
        """
        Closes a FinancialAccount. A FinancialAccount can only be closed if it has a zero balance, has no pending InboundTransfers, and has canceled all attached Issuing cards.
        """
        ...

    @overload
    def close(
        self, **params: Unpack["FinancialAccountCloseParams"]
    ) -> "FinancialAccount":
        """
        Closes a FinancialAccount. A FinancialAccount can only be closed if it has a zero balance, has no pending InboundTransfers, and has canceled all attached Issuing cards.
        """
        ...

    @class_method_variant("_cls_close")
    def close(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["FinancialAccountCloseParams"]
    ) -> "FinancialAccount":
        """
        Closes a FinancialAccount. A FinancialAccount can only be closed if it has a zero balance, has no pending InboundTransfers, and has canceled all attached Issuing cards.
        """
        return cast(
            "FinancialAccount",
            self._request(
                "post",
                "/v1/treasury/financial_accounts/{financial_account}/close".format(
                    financial_account=sanitize_id(self.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    async def _cls_close_async(
        cls,
        financial_account: str,
        **params: Unpack["FinancialAccountCloseParams"],
    ) -> "FinancialAccount":
        """
        Closes a FinancialAccount. A FinancialAccount can only be closed if it has a zero balance, has no pending InboundTransfers, and has canceled all attached Issuing cards.
        """
        return cast(
            "FinancialAccount",
            await cls._static_request_async(
                "post",
                "/v1/treasury/financial_accounts/{financial_account}/close".format(
                    financial_account=sanitize_id(financial_account)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def close_async(
        financial_account: str, **params: Unpack["FinancialAccountCloseParams"]
    ) -> "FinancialAccount":
        """
        Closes a FinancialAccount. A FinancialAccount can only be closed if it has a zero balance, has no pending InboundTransfers, and has canceled all attached Issuing cards.
        """
        ...

    @overload
    async def close_async(
        self, **params: Unpack["FinancialAccountCloseParams"]
    ) -> "FinancialAccount":
        """
        Closes a FinancialAccount. A FinancialAccount can only be closed if it has a zero balance, has no pending InboundTransfers, and has canceled all attached Issuing cards.
        """
        ...

    @class_method_variant("_cls_close_async")
    async def close_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["FinancialAccountCloseParams"]
    ) -> "FinancialAccount":
        """
        Closes a FinancialAccount. A FinancialAccount can only be closed if it has a zero balance, has no pending InboundTransfers, and has canceled all attached Issuing cards.
        """
        return cast(
            "FinancialAccount",
            await self._request_async(
                "post",
                "/v1/treasury/financial_accounts/{financial_account}/close".format(
                    financial_account=sanitize_id(self.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    def create(
        cls, **params: Unpack["FinancialAccountCreateParams"]
    ) -> "FinancialAccount":
        """
        Creates a new FinancialAccount. Each connected account can have up to three FinancialAccounts by default.
        """
        return cast(
            "FinancialAccount",
            cls._static_request(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    async def create_async(
        cls, **params: Unpack["FinancialAccountCreateParams"]
    ) -> "FinancialAccount":
        """
        Creates a new FinancialAccount. Each connected account can have up to three FinancialAccounts by default.
        """
        return cast(
            "FinancialAccount",
            await cls._static_request_async(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    def list(
        cls, **params: Unpack["FinancialAccountListParams"]
    ) -> ListObject["FinancialAccount"]:
        """
        Returns a list of FinancialAccounts.
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["FinancialAccountListParams"]
    ) -> ListObject["FinancialAccount"]:
        """
        Returns a list of FinancialAccounts.
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def modify(
        cls, id: str, **params: Unpack["FinancialAccountModifyParams"]
    ) -> "FinancialAccount":
        """
        Updates the details of a FinancialAccount.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "FinancialAccount",
            cls._static_request(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    async def modify_async(
        cls, id: str, **params: Unpack["FinancialAccountModifyParams"]
    ) -> "FinancialAccount":
        """
        Updates the details of a FinancialAccount.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "FinancialAccount",
            await cls._static_request_async(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["FinancialAccountRetrieveParams"]
    ) -> "FinancialAccount":
        """
        Retrieves the details of a FinancialAccount.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["FinancialAccountRetrieveParams"]
    ) -> "FinancialAccount":
        """
        Retrieves the details of a FinancialAccount.
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    @classmethod
    def _cls_retrieve_features(
        cls,
        financial_account: str,
        **params: Unpack["FinancialAccountRetrieveFeaturesParams"],
    ) -> "FinancialAccountFeatures":
        """
        Retrieves Features information associated with the FinancialAccount.
        """
        return cast(
            "FinancialAccountFeatures",
            cls._static_request(
                "get",
                "/v1/treasury/financial_accounts/{financial_account}/features".format(
                    financial_account=sanitize_id(financial_account)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    def retrieve_features(
        financial_account: str,
        **params: Unpack["FinancialAccountRetrieveFeaturesParams"],
    ) -> "FinancialAccountFeatures":
        """
        Retrieves Features information associated with the FinancialAccount.
        """
        ...

    @overload
    def retrieve_features(
        self, **params: Unpack["FinancialAccountRetrieveFeaturesParams"]
    ) -> "FinancialAccountFeatures":
        """
        Retrieves Features information associated with the FinancialAccount.
        """
        ...

    @class_method_variant("_cls_retrieve_features")
    def retrieve_features(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["FinancialAccountRetrieveFeaturesParams"]
    ) -> "FinancialAccountFeatures":
        """
        Retrieves Features information associated with the FinancialAccount.
        """
        return cast(
            "FinancialAccountFeatures",
            self._request(
                "get",
                "/v1/treasury/financial_accounts/{financial_account}/features".format(
                    financial_account=sanitize_id(self.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    async def _cls_retrieve_features_async(
        cls,
        financial_account: str,
        **params: Unpack["FinancialAccountRetrieveFeaturesParams"],
    ) -> "FinancialAccountFeatures":
        """
        Retrieves Features information associated with the FinancialAccount.
        """
        return cast(
            "FinancialAccountFeatures",
            await cls._static_request_async(
                "get",
                "/v1/treasury/financial_accounts/{financial_account}/features".format(
                    financial_account=sanitize_id(financial_account)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def retrieve_features_async(
        financial_account: str,
        **params: Unpack["FinancialAccountRetrieveFeaturesParams"],
    ) -> "FinancialAccountFeatures":
        """
        Retrieves Features information associated with the FinancialAccount.
        """
        ...

    @overload
    async def retrieve_features_async(
        self, **params: Unpack["FinancialAccountRetrieveFeaturesParams"]
    ) -> "FinancialAccountFeatures":
        """
        Retrieves Features information associated with the FinancialAccount.
        """
        ...

    @class_method_variant("_cls_retrieve_features_async")
    async def retrieve_features_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["FinancialAccountRetrieveFeaturesParams"]
    ) -> "FinancialAccountFeatures":
        """
        Retrieves Features information associated with the FinancialAccount.
        """
        return cast(
            "FinancialAccountFeatures",
            await self._request_async(
                "get",
                "/v1/treasury/financial_accounts/{financial_account}/features".format(
                    financial_account=sanitize_id(self.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    def _cls_update_features(
        cls,
        financial_account: str,
        **params: Unpack["FinancialAccountUpdateFeaturesParams"],
    ) -> "FinancialAccountFeatures":
        """
        Updates the Features associated with a FinancialAccount.
        """
        return cast(
            "FinancialAccountFeatures",
            cls._static_request(
                "post",
                "/v1/treasury/financial_accounts/{financial_account}/features".format(
                    financial_account=sanitize_id(financial_account)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    def update_features(
        financial_account: str,
        **params: Unpack["FinancialAccountUpdateFeaturesParams"],
    ) -> "FinancialAccountFeatures":
        """
        Updates the Features associated with a FinancialAccount.
        """
        ...

    @overload
    def update_features(
        self, **params: Unpack["FinancialAccountUpdateFeaturesParams"]
    ) -> "FinancialAccountFeatures":
        """
        Updates the Features associated with a FinancialAccount.
        """
        ...

    @class_method_variant("_cls_update_features")
    def update_features(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["FinancialAccountUpdateFeaturesParams"]
    ) -> "FinancialAccountFeatures":
        """
        Updates the Features associated with a FinancialAccount.
        """
        return cast(
            "FinancialAccountFeatures",
            self._request(
                "post",
                "/v1/treasury/financial_accounts/{financial_account}/features".format(
                    financial_account=sanitize_id(self.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    async def _cls_update_features_async(
        cls,
        financial_account: str,
        **params: Unpack["FinancialAccountUpdateFeaturesParams"],
    ) -> "FinancialAccountFeatures":
        """
        Updates the Features associated with a FinancialAccount.
        """
        return cast(
            "FinancialAccountFeatures",
            await cls._static_request_async(
                "post",
                "/v1/treasury/financial_accounts/{financial_account}/features".format(
                    financial_account=sanitize_id(financial_account)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def update_features_async(
        financial_account: str,
        **params: Unpack["FinancialAccountUpdateFeaturesParams"],
    ) -> "FinancialAccountFeatures":
        """
        Updates the Features associated with a FinancialAccount.
        """
        ...

    @overload
    async def update_features_async(
        self, **params: Unpack["FinancialAccountUpdateFeaturesParams"]
    ) -> "FinancialAccountFeatures":
        """
        Updates the Features associated with a FinancialAccount.
        """
        ...

    @class_method_variant("_cls_update_features_async")
    async def update_features_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["FinancialAccountUpdateFeaturesParams"]
    ) -> "FinancialAccountFeatures":
        """
        Updates the Features associated with a FinancialAccount.
        """
        return cast(
            "FinancialAccountFeatures",
            await self._request_async(
                "post",
                "/v1/treasury/financial_accounts/{financial_account}/features".format(
                    financial_account=sanitize_id(self.get("id"))
                ),
                params=params,
            ),
        )

    _inner_class_types = {
        "balance": Balance,
        "financial_addresses": FinancialAddress,
        "platform_restrictions": PlatformRestrictions,
        "status_details": StatusDetails,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from typing import ClassVar
from typing_extensions import Literal


class TaxDeductedAtSource(StripeObject):
    OBJECT_NAME: ClassVar[Literal["tax_deducted_at_source"]] = (
        "tax_deducted_at_source"
    )
    id: str
    """
    Unique identifier for the object.
    """
    object: Literal["tax_deducted_at_source"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    period_end: int
    """
    The end of the invoicing period. This TDS applies to Stripe fees collected during this invoicing period.
    """
    period_start: int
    """
    The start of the invoicing period. This TDS applies to Stripe fees collected during this invoicing period.
    """
    tax_deduction_account_number: str
    """
    The TAN that was supplied to Stripe when TDS was assessed
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.billing._credit_balance_summary import CreditBalanceSummary
    from stripe.params.billing._credit_balance_summary_retrieve_params import (
        CreditBalanceSummaryRetrieveParams,
    )


class CreditBalanceSummaryService(StripeService):
    def retrieve(
        self,
        params: "CreditBalanceSummaryRetrieveParams",
        options: Optional["RequestOptions"] = None,
    ) -> "CreditBalanceSummary":
        """
        Retrieves the credit balance summary for a customer.
        """
        return cast(
            "CreditBalanceSummary",
            self._request(
                "get",
                "/v1/billing/credit_balance_summary",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        params: "CreditBalanceSummaryRetrieveParams",
        options: Optional["RequestOptions"] = None,
    ) -> "CreditBalanceSummary":
        """
        Retrieves the credit balance summary for a customer.
        """
        return cast(
            "CreditBalanceSummary",
            await self._request_async(
                "get",
                "/v1/billing/credit_balance_summary",
                base_address="api",
                params=params,
                options=options,
            ),
        )



from stripe._singleton_api_resource import SingletonAPIResource


class TestSingletonAPIResource(object):
    class MySingleton(SingletonAPIResource):
        OBJECT_NAME = "mysingleton"

    def test_retrieve(self, http_client_mock):
        http_client_mock.stub_request(
            "get",
            path="/v1/mysingleton",
            rbody='{"single": "ton"}',
            rheaders={"request-id": "req_id"},
        )

        res = self.MySingleton.retrieve()

        http_client_mock.assert_requested("get", path="/v1/mysingleton")
        assert res.single == "ton"

        assert res.last_response is not None
        assert res.last_response.request_id == "req_id"



from typing_extensions import TYPE_CHECKING, Literal
from typing import Optional
import os
import warnings

# Stripe Python bindings
# API docs at http://stripe.com/docs/api
# Authors:
# Patrick Collison <patrick@stripe.com>
# Greg Brockman <gdb@stripe.com>
# Andrew Metcalf <andrew@stripe.com>

# Configuration variables
from stripe._api_version import _ApiVersion

from stripe._app_info import AppInfo as AppInfo
from stripe._version import VERSION as VERSION

# Constants
DEFAULT_API_BASE: str = "https://api.stripe.com"
DEFAULT_CONNECT_API_BASE: str = "https://connect.stripe.com"
DEFAULT_UPLOAD_API_BASE: str = "https://files.stripe.com"
DEFAULT_METER_EVENTS_API_BASE: str = "https://meter-events.stripe.com"


api_key: Optional[str] = None
client_id: Optional[str] = None
api_base: str = DEFAULT_API_BASE
connect_api_base: str = DEFAULT_CONNECT_API_BASE
upload_api_base: str = DEFAULT_UPLOAD_API_BASE
meter_events_api_base: str = DEFAULT_METER_EVENTS_API_BASE
api_version: str = _ApiVersion.CURRENT
verify_ssl_certs: bool = True
proxy: Optional[str] = None
default_http_client: Optional["HTTPClient"] = None
app_info: Optional[AppInfo] = None
enable_telemetry: bool = True
max_network_retries: int = 2
ca_bundle_path: str = os.path.join(
    os.path.dirname(__file__), "data", "ca-certificates.crt"
)

# Lazily initialized stripe.default_http_client
default_http_client = None
_default_proxy = None

from stripe._http_client import (
    new_default_http_client as new_default_http_client,
)


def ensure_default_http_client():
    if default_http_client:
        _warn_if_mismatched_proxy()
        return
    _init_default_http_client()


def _init_default_http_client():
    global _default_proxy
    global default_http_client

    # If the stripe.default_http_client has not been set by the user
    # yet, we'll set it here. This way, we aren't creating a new
    # HttpClient for every request.
    default_http_client = new_default_http_client(
        verify_ssl_certs=verify_ssl_certs, proxy=proxy
    )
    _default_proxy = proxy


def _warn_if_mismatched_proxy():
    global _default_proxy
    from stripe import proxy

    if proxy != _default_proxy:
        warnings.warn(
            "stripe.proxy was updated after sending a "
            "request - this is a no-op. To use a different proxy, "
            "set stripe.default_http_client to a new client "
            "configured with the proxy."
        )


# Set to either 'debug' or 'info', controls console logging
log: Optional[Literal["debug", "info"]] = None


# Sets some basic information about the running application that's sent along
# with API requests. Useful for plugin authors to identify their plugin when
# communicating with Stripe.
#
# Takes a name and optional version and plugin URL.
def set_app_info(
    name: str,
    partner_id: Optional[str] = None,
    url: Optional[str] = None,
    version: Optional[str] = None,
):
    global app_info
    app_info = {
        "name": name,
        "partner_id": partner_id,
        "url": url,
        "version": version,
    }


# The beginning of the section generated from our OpenAPI spec
from importlib import import_module

if TYPE_CHECKING:
    from stripe import (
        apps as apps,
        billing as billing,
        billing_portal as billing_portal,
        checkout as checkout,
        climate as climate,
        entitlements as entitlements,
        events as events,
        financial_connections as financial_connections,
        forwarding as forwarding,
        identity as identity,
        issuing as issuing,
        params as params,
        radar as radar,
        reporting as reporting,
        sigma as sigma,
        tax as tax,
        terminal as terminal,
        test_helpers as test_helpers,
        treasury as treasury,
        v2 as v2,
    )
    from stripe._account import Account as Account
    from stripe._account_capability_service import (
        AccountCapabilityService as AccountCapabilityService,
    )
    from stripe._account_external_account_service import (
        AccountExternalAccountService as AccountExternalAccountService,
    )
    from stripe._account_link import AccountLink as AccountLink
    from stripe._account_link_service import (
        AccountLinkService as AccountLinkService,
    )
    from stripe._account_login_link_service import (
        AccountLoginLinkService as AccountLoginLinkService,
    )
    from stripe._account_person_service import (
        AccountPersonService as AccountPersonService,
    )
    from stripe._account_service import AccountService as AccountService
    from stripe._account_session import AccountSession as AccountSession
    from stripe._account_session_service import (
        AccountSessionService as AccountSessionService,
    )
    from stripe._api_mode import ApiMode as ApiMode
    from stripe._api_resource import APIResource as APIResource
    from stripe._apple_pay_domain import ApplePayDomain as ApplePayDomain
    from stripe._apple_pay_domain_service import (
        ApplePayDomainService as ApplePayDomainService,
    )
    from stripe._application import Application as Application
    from stripe._application_fee import ApplicationFee as ApplicationFee
    from stripe._application_fee_refund import (
        ApplicationFeeRefund as ApplicationFeeRefund,
    )
    from stripe._application_fee_refund_service import (
        ApplicationFeeRefundService as ApplicationFeeRefundService,
    )
    from stripe._application_fee_service import (
        ApplicationFeeService as ApplicationFeeService,
    )
    from stripe._apps_service import AppsService as AppsService
    from stripe._balance import Balance as Balance
    from stripe._balance_service import BalanceService as BalanceService
    from stripe._balance_settings import BalanceSettings as BalanceSettings
    from stripe._balance_settings_service import (
        BalanceSettingsService as BalanceSettingsService,
    )
    from stripe._balance_transaction import (
        BalanceTransaction as BalanceTransaction,
    )
    from stripe._balance_transaction_service import (
        BalanceTransactionService as BalanceTransactionService,
    )
    from stripe._bank_account import BankAccount as BankAccount
    from stripe._base_address import BaseAddress as BaseAddress
    from stripe._billing_portal_service import (
        BillingPortalService as BillingPortalService,
    )
    from stripe._billing_service import BillingService as BillingService
    from stripe._capability import Capability as Capability
    from stripe._card import Card as Card
    from stripe._cash_balance import CashBalance as CashBalance
    from stripe._charge import Charge as Charge
    from stripe._charge_service import ChargeService as ChargeService
    from stripe._checkout_service import CheckoutService as CheckoutService
    from stripe._climate_service import ClimateService as ClimateService
    from stripe._confirmation_token import (
        ConfirmationToken as ConfirmationToken,
    )
    from stripe._confirmation_token_service import (
        ConfirmationTokenService as ConfirmationTokenService,
    )
    from stripe._connect_collection_transfer import (
        ConnectCollectionTransfer as ConnectCollectionTransfer,
    )
    from stripe._country_spec import CountrySpec as CountrySpec
    from stripe._country_spec_service import (
        CountrySpecService as CountrySpecService,
    )
    from stripe._coupon import Coupon as Coupon
    from stripe._coupon_service import CouponService as CouponService
    from stripe._createable_api_resource import (
        CreateableAPIResource as CreateableAPIResource,
    )
    from stripe._credit_note import CreditNote as CreditNote
    from stripe._credit_note_line_item import (
        CreditNoteLineItem as CreditNoteLineItem,
    )
    from stripe._credit_note_line_item_service import (
        CreditNoteLineItemService as CreditNoteLineItemService,
    )
    from stripe._credit_note_preview_lines_service import (
        CreditNotePreviewLinesService as CreditNotePreviewLinesService,
    )
    from stripe._credit_note_service import (
        CreditNoteService as CreditNoteService,
    )
    from stripe._custom_method import custom_method as custom_method
    from stripe._customer import Customer as Customer
    from stripe._customer_balance_transaction import (
        CustomerBalanceTransaction as CustomerBalanceTransaction,
    )
    from stripe._customer_balance_transaction_service import (
        CustomerBalanceTransactionService as CustomerBalanceTransactionService,
    )
    from stripe._customer_cash_balance_service import (
        CustomerCashBalanceService as CustomerCashBalanceService,
    )
    from stripe._customer_cash_balance_transaction import (
        CustomerCashBalanceTransaction as CustomerCashBalanceTransaction,
    )
    from stripe._customer_cash_balance_transaction_service import (
        CustomerCashBalanceTransactionService as CustomerCashBalanceTransactionService,
    )
    from stripe._customer_funding_instructions_service import (
        CustomerFundingInstructionsService as CustomerFundingInstructionsService,
    )
    from stripe._customer_payment_method_service import (
        CustomerPaymentMethodService as CustomerPaymentMethodService,
    )
    from stripe._customer_payment_source_service import (
        CustomerPaymentSourceService as CustomerPaymentSourceService,
    )
    from stripe._customer_service import CustomerService as CustomerService
    from stripe._customer_session import CustomerSession as CustomerSession
    from stripe._customer_session_service import (
        CustomerSessionService as CustomerSessionService,
    )
    from stripe._customer_tax_id_service import (
        CustomerTaxIdService as CustomerTaxIdService,
    )
    from stripe._deletable_api_resource import (
        DeletableAPIResource as DeletableAPIResource,
    )
    from stripe._discount import Discount as Discount
    from stripe._dispute import Dispute as Dispute
    from stripe._dispute_service import DisputeService as DisputeService
    from stripe._entitlements_service import (
        EntitlementsService as EntitlementsService,
    )
    from stripe._ephemeral_key import EphemeralKey as EphemeralKey
    from stripe._ephemeral_key_service import (
        EphemeralKeyService as EphemeralKeyService,
    )
    from stripe._error import (
        APIConnectionError as APIConnectionError,
        APIError as APIError,
        AuthenticationError as AuthenticationError,
        CardError as CardError,
        IdempotencyError as IdempotencyError,
        InvalidRequestError as InvalidRequestError,
        PermissionError as PermissionError,
        RateLimitError as RateLimitError,
        SignatureVerificationError as SignatureVerificationError,
        StripeError as StripeError,
        StripeErrorWithParamCode as StripeErrorWithParamCode,
        TemporarySessionExpiredError as TemporarySessionExpiredError,
    )
    from stripe._error_object import (
        ErrorObject as ErrorObject,
        OAuthErrorObject as OAuthErrorObject,
    )
    from stripe._event import Event as Event
    from stripe._event_service import EventService as EventService
    from stripe._exchange_rate import ExchangeRate as ExchangeRate
    from stripe._exchange_rate_service import (
        ExchangeRateService as ExchangeRateService,
    )
    from stripe._file import File as File
    from stripe._file_link import FileLink as FileLink
    from stripe._file_link_service import FileLinkService as FileLinkService
    from stripe._file_service import FileService as FileService
    from stripe._financial_connections_service import (
        FinancialConnectionsService as FinancialConnectionsService,
    )
    from stripe._forwarding_service import (
        ForwardingService as ForwardingService,
    )
    from stripe._funding_instructions import (
        FundingInstructions as FundingInstructions,
    )
    from stripe._http_client import (
        AIOHTTPClient as AIOHTTPClient,
        HTTPClient as HTTPClient,
        HTTPXClient as HTTPXClient,
        PycurlClient as PycurlClient,
        RequestsClient as RequestsClient,
        UrlFetchClient as UrlFetchClient,
        UrllibClient as UrllibClient,
    )
    from stripe._identity_service import IdentityService as IdentityService
    from stripe._invoice import Invoice as Invoice
    from stripe._invoice_item import InvoiceItem as InvoiceItem
    from stripe._invoice_item_service import (
        InvoiceItemService as InvoiceItemService,
    )
    from stripe._invoice_line_item import InvoiceLineItem as InvoiceLineItem
    from stripe._invoice_line_item_service import (
        InvoiceLineItemService as InvoiceLineItemService,
    )
    from stripe._invoice_payment import InvoicePayment as InvoicePayment
    from stripe._invoice_payment_service import (
        InvoicePaymentService as InvoicePaymentService,
    )
    from stripe._invoice_rendering_template import (
        InvoiceRenderingTemplate as InvoiceRenderingTemplate,
    )
    from stripe._invoice_rendering_template_service import (
        InvoiceRenderingTemplateService as InvoiceRenderingTemplateService,
    )
    from stripe._invoice_service import InvoiceService as InvoiceService
    from stripe._issuing_service import IssuingService as IssuingService
    from stripe._line_item import LineItem as LineItem
    from stripe._list_object import ListObject as ListObject
    from stripe._listable_api_resource import (
        ListableAPIResource as ListableAPIResource,
    )
    from stripe._login_link import LoginLink as LoginLink
    from stripe._mandate import Mandate as Mandate
    from stripe._mandate_service import MandateService as MandateService
    from stripe._nested_resource_class_methods import (
        nested_resource_class_methods as nested_resource_class_methods,
    )
    from stripe._oauth import OAuth as OAuth
    from stripe._oauth_service import OAuthService as OAuthService
    from stripe._payment_attempt_record import (
        PaymentAttemptRecord as PaymentAttemptRecord,
    )
    from stripe._payment_attempt_record_service import (
        PaymentAttemptRecordService as PaymentAttemptRecordService,
    )
    from stripe._payment_intent import PaymentIntent as PaymentIntent
    from stripe._payment_intent_amount_details_line_item import (
        PaymentIntentAmountDetailsLineItem as PaymentIntentAmountDetailsLineItem,
    )
    from stripe._payment_intent_amount_details_line_item_service import (
        PaymentIntentAmountDetailsLineItemService as PaymentIntentAmountDetailsLineItemService,
    )
    from stripe._payment_intent_service import (
        PaymentIntentService as PaymentIntentService,
    )
    from stripe._payment_link import PaymentLink as PaymentLink
    from stripe._payment_link_line_item_service import (
        PaymentLinkLineItemService as PaymentLinkLineItemService,
    )
    from stripe._payment_link_service import (
        PaymentLinkService as PaymentLinkService,
    )
    from stripe._payment_method import PaymentMethod as PaymentMethod
    from stripe._payment_method_configuration import (
        PaymentMethodConfiguration as PaymentMethodConfiguration,
    )
    from stripe._payment_method_configuration_service import (
        PaymentMethodConfigurationService as PaymentMethodConfigurationService,
    )
    from stripe._payment_method_domain import (
        PaymentMethodDomain as PaymentMethodDomain,
    )
    from stripe._payment_method_domain_service import (
        PaymentMethodDomainService as PaymentMethodDomainService,
    )
    from stripe._payment_method_service import (
        PaymentMethodService as PaymentMethodService,
    )
    from stripe._payment_record import PaymentRecord as PaymentRecord
    from stripe._payment_record_service import (
        PaymentRecordService as PaymentRecordService,
    )
    from stripe._payout import Payout as Payout
    from stripe._payout_service import PayoutService as PayoutService
    from stripe._person import Person as Person
    from stripe._plan import Plan as Plan
    from stripe._plan_service import PlanService as PlanService
    from stripe._price import Price as Price
    from stripe._price_service import PriceService as PriceService
    from stripe._product import Product as Product
    from stripe._product_feature import ProductFeature as ProductFeature
    from stripe._product_feature_service import (
        ProductFeatureService as ProductFeatureService,
    )
    from stripe._product_service import ProductService as ProductService
    from stripe._promotion_code import PromotionCode as PromotionCode
    from stripe._promotion_code_service import (
        PromotionCodeService as PromotionCodeService,
    )
    from stripe._quote import Quote as Quote
    from stripe._quote_computed_upfront_line_items_service import (
        QuoteComputedUpfrontLineItemsService as QuoteComputedUpfrontLineItemsService,
    )
    from stripe._quote_line_item_service import (
        QuoteLineItemService as QuoteLineItemService,
    )
    from stripe._quote_service import QuoteService as QuoteService
    from stripe._radar_service import RadarService as RadarService
    from stripe._refund import Refund as Refund
    from stripe._refund_service import RefundService as RefundService
    from stripe._reporting_service import ReportingService as ReportingService
    from stripe._request_options import RequestOptions as RequestOptions
    from stripe._requestor_options import RequestorOptions as RequestorOptions
    from stripe._reserve_transaction import (
        ReserveTransaction as ReserveTransaction,
    )
    from stripe._reversal import Reversal as Reversal
    from stripe._review import Review as Review
    from stripe._review_service import ReviewService as ReviewService
    from stripe._search_result_object import (
        SearchResultObject as SearchResultObject,
    )
    from stripe._searchable_api_resource import (
        SearchableAPIResource as SearchableAPIResource,
    )
    from stripe._setup_attempt import SetupAttempt as SetupAttempt
    from stripe._setup_attempt_service import (
        SetupAttemptService as SetupAttemptService,
    )
    from stripe._setup_intent import SetupIntent as SetupIntent
    from stripe._setup_intent_service import (
        SetupIntentService as SetupIntentService,
    )
    from stripe._shipping_rate import ShippingRate as ShippingRate
    from stripe._shipping_rate_service import (
        ShippingRateService as ShippingRateService,
    )
    from stripe._sigma_service import SigmaService as SigmaService
    from stripe._singleton_api_resource import (
        SingletonAPIResource as SingletonAPIResource,
    )
    from stripe._source import Source as Source
    from stripe._source_mandate_notification import (
        SourceMandateNotification as SourceMandateNotification,
    )
    from stripe._source_service import SourceService as SourceService
    from stripe._source_transaction import (
        SourceTransaction as SourceTransaction,
    )
    from stripe._source_transaction_service import (
        SourceTransactionService as SourceTransactionService,
    )
    from stripe._stripe_client import StripeClient as StripeClient
    from stripe._stripe_context import StripeContext as StripeContext
    from stripe._stripe_object import StripeObject as StripeObject
    from stripe._stripe_response import (
        StripeResponse as StripeResponse,
        StripeResponseBase as StripeResponseBase,
        StripeStreamResponse as StripeStreamResponse,
        StripeStreamResponseAsync as StripeStreamResponseAsync,
    )
    from stripe._subscription import Subscription as Subscription
    from stripe._subscription_item import SubscriptionItem as SubscriptionItem
    from stripe._subscription_item_service import (
        SubscriptionItemService as SubscriptionItemService,
    )
    from stripe._subscription_schedule import (
        SubscriptionSchedule as SubscriptionSchedule,
    )
    from stripe._subscription_schedule_service import (
        SubscriptionScheduleService as SubscriptionScheduleService,
    )
    from stripe._subscription_service import (
        SubscriptionService as SubscriptionService,
    )
    from stripe._tax_code import TaxCode as TaxCode
    from stripe._tax_code_service import TaxCodeService as TaxCodeService
    from stripe._tax_deducted_at_source import (
        TaxDeductedAtSource as TaxDeductedAtSource,
    )
    from stripe._tax_id import TaxId as TaxId
    from stripe._tax_id_service import TaxIdService as TaxIdService
    from stripe._tax_rate import TaxRate as TaxRate
    from stripe._tax_rate_service import TaxRateService as TaxRateService
    from stripe._tax_service import TaxService as TaxService
    from stripe._terminal_service import TerminalService as TerminalService
    from stripe._test_helpers import (
        APIResourceTestHelpers as APIResourceTestHelpers,
    )
    from stripe._test_helpers_service import (
        TestHelpersService as TestHelpersService,
    )
    from stripe._token import Token as Token
    from stripe._token_service import TokenService as TokenService
    from stripe._topup import Topup as Topup
    from stripe._topup_service import TopupService as TopupService
    from stripe._transfer import Transfer as Transfer
    from stripe._transfer_reversal_service import (
        TransferReversalService as TransferReversalService,
    )
    from stripe._transfer_service import TransferService as TransferService
    from stripe._treasury_service import TreasuryService as TreasuryService
    from stripe._updateable_api_resource import (
        UpdateableAPIResource as UpdateableAPIResource,
    )
    from stripe._util import (
        convert_to_stripe_object as convert_to_stripe_object,
    )
    from stripe._v1_services import V1Services as V1Services
    from stripe._v2_services import V2Services as V2Services
    from stripe._verify_mixin import VerifyMixin as VerifyMixin
    from stripe._webhook import (
        Webhook as Webhook,
        WebhookSignature as WebhookSignature,
    )
    from stripe._webhook_endpoint import WebhookEndpoint as WebhookEndpoint
    from stripe._webhook_endpoint_service import (
        WebhookEndpointService as WebhookEndpointService,
    )

# name -> (import_target, is_submodule)
_import_map = {
    "apps": ("stripe.apps", True),
    "billing": ("stripe.billing", True),
    "billing_portal": ("stripe.billing_portal", True),
    "checkout": ("stripe.checkout", True),
    "climate": ("stripe.climate", True),
    "entitlements": ("stripe.entitlements", True),
    "events": ("stripe.events", True),
    "financial_connections": ("stripe.financial_connections", True),
    "forwarding": ("stripe.forwarding", True),
    "identity": ("stripe.identity", True),
    "issuing": ("stripe.issuing", True),
    "params": ("stripe.params", True),
    "radar": ("stripe.radar", True),
    "reporting": ("stripe.reporting", True),
    "sigma": ("stripe.sigma", True),
    "tax": ("stripe.tax", True),
    "terminal": ("stripe.terminal", True),
    "test_helpers": ("stripe.test_helpers", True),
    "treasury": ("stripe.treasury", True),
    "v2": ("stripe.v2", True),
    "Account": ("stripe._account", False),
    "AccountCapabilityService": ("stripe._account_capability_service", False),
    "AccountExternalAccountService": (
        "stripe._account_external_account_service",
        False,
    ),
    "AccountLink": ("stripe._account_link", False),
    "AccountLinkService": ("stripe._account_link_service", False),
    "AccountLoginLinkService": ("stripe._account_login_link_service", False),
    "AccountPersonService": ("stripe._account_person_service", False),
    "AccountService": ("stripe._account_service", False),
    "AccountSession": ("stripe._account_session", False),
    "AccountSessionService": ("stripe._account_session_service", False),
    "ApiMode": ("stripe._api_mode", False),
    "APIResource": ("stripe._api_resource", False),
    "ApplePayDomain": ("stripe._apple_pay_domain", False),
    "ApplePayDomainService": ("stripe._apple_pay_domain_service", False),
    "Application": ("stripe._application", False),
    "ApplicationFee": ("stripe._application_fee", False),
    "ApplicationFeeRefund": ("stripe._application_fee_refund", False),
    "ApplicationFeeRefundService": (
        "stripe._application_fee_refund_service",
        False,
    ),
    "ApplicationFeeService": ("stripe._application_fee_service", False),
    "AppsService": ("stripe._apps_service", False),
    "Balance": ("stripe._balance", False),
    "BalanceService": ("stripe._balance_service", False),
    "BalanceSettings": ("stripe._balance_settings", False),
    "BalanceSettingsService": ("stripe._balance_settings_service", False),
    "BalanceTransaction": ("stripe._balance_transaction", False),
    "BalanceTransactionService": (
        "stripe._balance_transaction_service",
        False,
    ),
    "BankAccount": ("stripe._bank_account", False),
    "BaseAddress": ("stripe._base_address", False),
    "BillingPortalService": ("stripe._billing_portal_service", False),
    "BillingService": ("stripe._billing_service", False),
    "Capability": ("stripe._capability", False),
    "Card": ("stripe._card", False),
    "CashBalance": ("stripe._cash_balance", False),
    "Charge": ("stripe._charge", False),
    "ChargeService": ("stripe._charge_service", False),
    "CheckoutService": ("stripe._checkout_service", False),
    "ClimateService": ("stripe._climate_service", False),
    "ConfirmationToken": ("stripe._confirmation_token", False),
    "ConfirmationTokenService": ("stripe._confirmation_token_service", False),
    "ConnectCollectionTransfer": (
        "stripe._connect_collection_transfer",
        False,
    ),
    "CountrySpec": ("stripe._country_spec", False),
    "CountrySpecService": ("stripe._country_spec_service", False),
    "Coupon": ("stripe._coupon", False),
    "CouponService": ("stripe._coupon_service", False),
    "CreateableAPIResource": ("stripe._createable_api_resource", False),
    "CreditNote": ("stripe._credit_note", False),
    "CreditNoteLineItem": ("stripe._credit_note_line_item", False),
    "CreditNoteLineItemService": (
        "stripe._credit_note_line_item_service",
        False,
    ),
    "CreditNotePreviewLinesService": (
        "stripe._credit_note_preview_lines_service",
        False,
    ),
    "CreditNoteService": ("stripe._credit_note_service", False),
    "custom_method": ("stripe._custom_method", False),
    "Customer": ("stripe._customer", False),
    "CustomerBalanceTransaction": (
        "stripe._customer_balance_transaction",
        False,
    ),
    "CustomerBalanceTransactionService": (
        "stripe._customer_balance_transaction_service",
        False,
    ),
    "CustomerCashBalanceService": (
        "stripe._customer_cash_balance_service",
        False,
    ),
    "CustomerCashBalanceTransaction": (
        "stripe._customer_cash_balance_transaction",
        False,
    ),
    "CustomerCashBalanceTransactionService": (
        "stripe._customer_cash_balance_transaction_service",
        False,
    ),
    "CustomerFundingInstructionsService": (
        "stripe._customer_funding_instructions_service",
        False,
    ),
    "CustomerPaymentMethodService": (
        "stripe._customer_payment_method_service",
        False,
    ),
    "CustomerPaymentSourceService": (
        "stripe._customer_payment_source_service",
        False,
    ),
    "CustomerService": ("stripe._customer_service", False),
    "CustomerSession": ("stripe._customer_session", False),
    "CustomerSessionService": ("stripe._customer_session_service", False),
    "CustomerTaxIdService": ("stripe._customer_tax_id_service", False),
    "DeletableAPIResource": ("stripe._deletable_api_resource", False),
    "Discount": ("stripe._discount", False),
    "Dispute": ("stripe._dispute", False),
    "DisputeService": ("stripe._dispute_service", False),
    "EntitlementsService": ("stripe._entitlements_service", False),
    "EphemeralKey": ("stripe._ephemeral_key", False),
    "EphemeralKeyService": ("stripe._ephemeral_key_service", False),
    "APIConnectionError": ("stripe._error", False),
    "APIError": ("stripe._error", False),
    "AuthenticationError": ("stripe._error", False),
    "CardError": ("stripe._error", False),
    "IdempotencyError": ("stripe._error", False),
    "InvalidRequestError": ("stripe._error", False),
    "PermissionError": ("stripe._error", False),
    "RateLimitError": ("stripe._error", False),
    "SignatureVerificationError": ("stripe._error", False),
    "StripeError": ("stripe._error", False),
    "StripeErrorWithParamCode": ("stripe._error", False),
    "TemporarySessionExpiredError": ("stripe._error", False),
    "ErrorObject": ("stripe._error_object", False),
    "OAuthErrorObject": ("stripe._error_object", False),
    "Event": ("stripe._event", False),
    "EventService": ("stripe._event_service", False),
    "ExchangeRate": ("stripe._exchange_rate", False),
    "ExchangeRateService": ("stripe._exchange_rate_service", False),
    "File": ("stripe._file", False),
    "FileLink": ("stripe._file_link", False),
    "FileLinkService": ("stripe._file_link_service", False),
    "FileService": ("stripe._file_service", False),
    "FinancialConnectionsService": (
        "stripe._financial_connections_service",
        False,
    ),
    "ForwardingService": ("stripe._forwarding_service", False),
    "FundingInstructions": ("stripe._funding_instructions", False),
    "AIOHTTPClient": ("stripe._http_client", False),
    "HTTPClient": ("stripe._http_client", False),
    "HTTPXClient": ("stripe._http_client", False),
    "PycurlClient": ("stripe._http_client", False),
    "RequestsClient": ("stripe._http_client", False),
    "UrlFetchClient": ("stripe._http_client", False),
    "UrllibClient": ("stripe._http_client", False),
    "IdentityService": ("stripe._identity_service", False),
    "Invoice": ("stripe._invoice", False),
    "InvoiceItem": ("stripe._invoice_item", False),
    "InvoiceItemService": ("stripe._invoice_item_service", False),
    "InvoiceLineItem": ("stripe._invoice_line_item", False),
    "InvoiceLineItemService": ("stripe._invoice_line_item_service", False),
    "InvoicePayment": ("stripe._invoice_payment", False),
    "InvoicePaymentService": ("stripe._invoice_payment_service", False),
    "InvoiceRenderingTemplate": ("stripe._invoice_rendering_template", False),
    "InvoiceRenderingTemplateService": (
        "stripe._invoice_rendering_template_service",
        False,
    ),
    "InvoiceService": ("stripe._invoice_service", False),
    "IssuingService": ("stripe._issuing_service", False),
    "LineItem": ("stripe._line_item", False),
    "ListObject": ("stripe._list_object", False),
    "ListableAPIResource": ("stripe._listable_api_resource", False),
    "LoginLink": ("stripe._login_link", False),
    "Mandate": ("stripe._mandate", False),
    "MandateService": ("stripe._mandate_service", False),
    "nested_resource_class_methods": (
        "stripe._nested_resource_class_methods",
        False,
    ),
    "OAuth": ("stripe._oauth", False),
    "OAuthService": ("stripe._oauth_service", False),
    "PaymentAttemptRecord": ("stripe._payment_attempt_record", False),
    "PaymentAttemptRecordService": (
        "stripe._payment_attempt_record_service",
        False,
    ),
    "PaymentIntent": ("stripe._payment_intent", False),
    "PaymentIntentAmountDetailsLineItem": (
        "stripe._payment_intent_amount_details_line_item",
        False,
    ),
    "PaymentIntentAmountDetailsLineItemService": (
        "stripe._payment_intent_amount_details_line_item_service",
        False,
    ),
    "PaymentIntentService": ("stripe._payment_intent_service", False),
    "PaymentLink": ("stripe._payment_link", False),
    "PaymentLinkLineItemService": (
        "stripe._payment_link_line_item_service",
        False,
    ),
    "PaymentLinkService": ("stripe._payment_link_service", False),
    "PaymentMethod": ("stripe._payment_method", False),
    "PaymentMethodConfiguration": (
        "stripe._payment_method_configuration",
        False,
    ),
    "PaymentMethodConfigurationService": (
        "stripe._payment_method_configuration_service",
        False,
    ),
    "PaymentMethodDomain": ("stripe._payment_method_domain", False),
    "PaymentMethodDomainService": (
        "stripe._payment_method_domain_service",
        False,
    ),
    "PaymentMethodService": ("stripe._payment_method_service", False),
    "PaymentRecord": ("stripe._payment_record", False),
    "PaymentRecordService": ("stripe._payment_record_service", False),
    "Payout": ("stripe._payout", False),
    "PayoutService": ("stripe._payout_service", False),
    "Person": ("stripe._person", False),
    "Plan": ("stripe._plan", False),
    "PlanService": ("stripe._plan_service", False),
    "Price": ("stripe._price", False),
    "PriceService": ("stripe._price_service", False),
    "Product": ("stripe._product", False),
    "ProductFeature": ("stripe._product_feature", False),
    "ProductFeatureService": ("stripe._product_feature_service", False),
    "ProductService": ("stripe._product_service", False),
    "PromotionCode": ("stripe._promotion_code", False),
    "PromotionCodeService": ("stripe._promotion_code_service", False),
    "Quote": ("stripe._quote", False),
    "QuoteComputedUpfrontLineItemsService": (
        "stripe._quote_computed_upfront_line_items_service",
        False,
    ),
    "QuoteLineItemService": ("stripe._quote_line_item_service", False),
    "QuoteService": ("stripe._quote_service", False),
    "RadarService": ("stripe._radar_service", False),
    "Refund": ("stripe._refund", False),
    "RefundService": ("stripe._refund_service", False),
    "ReportingService": ("stripe._reporting_service", False),
    "RequestOptions": ("stripe._request_options", False),
    "RequestorOptions": ("stripe._requestor_options", False),
    "ReserveTransaction": ("stripe._reserve_transaction", False),
    "Reversal": ("stripe._reversal", False),
    "Review": ("stripe._review", False),
    "ReviewService": ("stripe._review_service", False),
    "SearchResultObject": ("stripe._search_result_object", False),
    "SearchableAPIResource": ("stripe._searchable_api_resource", False),
    "SetupAttempt": ("stripe._setup_attempt", False),
    "SetupAttemptService": ("stripe._setup_attempt_service", False),
    "SetupIntent": ("stripe._setup_intent", False),
    "SetupIntentService": ("stripe._setup_intent_service", False),
    "ShippingRate": ("stripe._shipping_rate", False),
    "ShippingRateService": ("stripe._shipping_rate_service", False),
    "SigmaService": ("stripe._sigma_service", False),
    "SingletonAPIResource": ("stripe._singleton_api_resource", False),
    "Source": ("stripe._source", False),
    "SourceMandateNotification": (
        "stripe._source_mandate_notification",
        False,
    ),
    "SourceService": ("stripe._source_service", False),
    "SourceTransaction": ("stripe._source_transaction", False),
    "SourceTransactionService": ("stripe._source_transaction_service", False),
    "StripeClient": ("stripe._stripe_client", False),
    "StripeContext": ("stripe._stripe_context", False),
    "StripeObject": ("stripe._stripe_object", False),
    "StripeResponse": ("stripe._stripe_response", False),
    "StripeResponseBase": ("stripe._stripe_response", False),
    "StripeStreamResponse": ("stripe._stripe_response", False),
    "StripeStreamResponseAsync": ("stripe._stripe_response", False),
    "Subscription": ("stripe._subscription", False),
    "SubscriptionItem": ("stripe._subscription_item", False),
    "SubscriptionItemService": ("stripe._subscription_item_service", False),
    "SubscriptionSchedule": ("stripe._subscription_schedule", False),
    "SubscriptionScheduleService": (
        "stripe._subscription_schedule_service",
        False,
    ),
    "SubscriptionService": ("stripe._subscription_service", False),
    "TaxCode": ("stripe._tax_code", False),
    "TaxCodeService": ("stripe._tax_code_service", False),
    "TaxDeductedAtSource": ("stripe._tax_deducted_at_source", False),
    "TaxId": ("stripe._tax_id", False),
    "TaxIdService": ("stripe._tax_id_service", False),
    "TaxRate": ("stripe._tax_rate", False),
    "TaxRateService": ("stripe._tax_rate_service", False),
    "TaxService": ("stripe._tax_service", False),
    "TerminalService": ("stripe._terminal_service", False),
    "APIResourceTestHelpers": ("stripe._test_helpers", False),
    "TestHelpersService": ("stripe._test_helpers_service", False),
    "Token": ("stripe._token", False),
    "TokenService": ("stripe._token_service", False),
    "Topup": ("stripe._topup", False),
    "TopupService": ("stripe._topup_service", False),
    "Transfer": ("stripe._transfer", False),
    "TransferReversalService": ("stripe._transfer_reversal_service", False),
    "TransferService": ("stripe._transfer_service", False),
    "TreasuryService": ("stripe._treasury_service", False),
    "UpdateableAPIResource": ("stripe._updateable_api_resource", False),
    "convert_to_stripe_object": ("stripe._util", False),
    "V1Services": ("stripe._v1_services", False),
    "V2Services": ("stripe._v2_services", False),
    "VerifyMixin": ("stripe._verify_mixin", False),
    "Webhook": ("stripe._webhook", False),
    "WebhookSignature": ("stripe._webhook", False),
    "WebhookEndpoint": ("stripe._webhook_endpoint", False),
    "WebhookEndpointService": ("stripe._webhook_endpoint_service", False),
}
if not TYPE_CHECKING:

    def __getattr__(name):
        try:
            target, is_submodule = _import_map[name]
            module = import_module(target)
            if is_submodule:
                return module

            return getattr(
                module,
                name,
            )
        except KeyError:
            raise AttributeError()

# The end of the section generated from our OpenAPI spec



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe.climate._supplier import Supplier
    from stripe.params.climate._supplier_list_params import SupplierListParams
    from stripe.params.climate._supplier_retrieve_params import (
        SupplierRetrieveParams,
    )


class SupplierService(StripeService):
    def list(
        self,
        params: Optional["SupplierListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Supplier]":
        """
        Lists all available Climate supplier objects.
        """
        return cast(
            "ListObject[Supplier]",
            self._request(
                "get",
                "/v1/climate/suppliers",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["SupplierListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Supplier]":
        """
        Lists all available Climate supplier objects.
        """
        return cast(
            "ListObject[Supplier]",
            await self._request_async(
                "get",
                "/v1/climate/suppliers",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        supplier: str,
        params: Optional["SupplierRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Supplier":
        """
        Retrieves a Climate supplier object.
        """
        return cast(
            "Supplier",
            self._request(
                "get",
                "/v1/climate/suppliers/{supplier}".format(
                    supplier=sanitize_id(supplier),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        supplier: str,
        params: Optional["SupplierRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Supplier":
        """
        Retrieves a Climate supplier object.
        """
        return cast(
            "Supplier",
            await self._request_async(
                "get",
                "/v1/climate/suppliers/{supplier}".format(
                    supplier=sanitize_id(supplier),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._event import Event
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe.params._event_list_params import EventListParams
    from stripe.params._event_retrieve_params import EventRetrieveParams


class EventService(StripeService):
    def list(
        self,
        params: Optional["EventListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Event]":
        """
        List events, going back up to 30 days. Each event data is rendered according to Stripe API version at its creation time, specified in [event object](https://docs.stripe.com/api/events/object) api_version attribute (not according to your current Stripe API version or Stripe-Version header).
        """
        return cast(
            "ListObject[Event]",
            self._request(
                "get",
                "/v1/events",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["EventListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Event]":
        """
        List events, going back up to 30 days. Each event data is rendered according to Stripe API version at its creation time, specified in [event object](https://docs.stripe.com/api/events/object) api_version attribute (not according to your current Stripe API version or Stripe-Version header).
        """
        return cast(
            "ListObject[Event]",
            await self._request_async(
                "get",
                "/v1/events",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: Optional["EventRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Event":
        """
        Retrieves the details of an event if it was created in the last 30 days. Supply the unique identifier of the event, which you might have received in a webhook.
        """
        return cast(
            "Event",
            self._request(
                "get",
                "/v1/events/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["EventRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Event":
        """
        Retrieves the details of an event if it was created in the last 30 days. Supply the unique identifier of the event, which you might have received in a webhook.
        """
        return cast(
            "Event",
            await self._request_async(
                "get",
                "/v1/events/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import Dict, List
from typing_extensions import Literal, NotRequired


class PaymentRecordReportPaymentAttemptGuaranteedParams(RequestOptions):
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    guaranteed_at: int
    """
    When the reported payment was guaranteed. Measured in seconds since the Unix epoch.
    """
    metadata: NotRequired["Literal['']|Dict[str, str]"]



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import List
from typing_extensions import NotRequired, TypedDict


class ChargeCaptureParams(RequestOptions):
    amount: NotRequired[int]
    """
    The amount to capture, which must be less than or equal to the original amount.
    """
    application_fee: NotRequired[int]
    """
    An application fee to add on to this charge.
    """
    application_fee_amount: NotRequired[int]
    """
    An application fee amount to add on to this charge, which must be less than or equal to the original amount.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    receipt_email: NotRequired[str]
    """
    The email address to send this charge's receipt to. This will override the previously-specified email address for this charge, if one was set. Receipts will not be sent in test mode.
    """
    statement_descriptor: NotRequired[str]
    """
    For a non-card charge, text that appears on the customer's statement as the statement descriptor. This value overrides the account's default statement descriptor. For information about requirements, including the 22-character limit, see [the Statement Descriptor docs](https://docs.stripe.com/get-started/account/statement-descriptors).

    For a card charge, this value is ignored unless you don't specify a `statement_descriptor_suffix`, in which case this value is used as the suffix.
    """
    statement_descriptor_suffix: NotRequired[str]
    """
    Provides information about a card charge. Concatenated to the account's [statement descriptor prefix](https://docs.stripe.com/get-started/account/statement-descriptors#static) to form the complete statement descriptor that appears on the customer's statement. If the account has no prefix value, the suffix is concatenated to the account's statement descriptor.
    """
    transfer_data: NotRequired["ChargeCaptureParamsTransferData"]
    """
    An optional dictionary including the account to automatically transfer to as part of a destination charge. [See the Connect documentation](https://stripe.com/docs/connect/destination-charges) for details.
    """
    transfer_group: NotRequired[str]
    """
    A string that identifies this transaction as part of a group. `transfer_group` may only be provided if it has not been set. See the [Connect documentation](https://stripe.com/docs/connect/separate-charges-and-transfers#transfer-options) for details.
    """


class ChargeCaptureParamsTransferData(TypedDict):
    amount: NotRequired[int]
    """
    The amount transferred to the destination account, if specified. By default, the entire charge amount is transferred to the destination account.
    """



from stripe import _util
from stripe._api_resource import APIResource
from urllib.parse import quote_plus
from typing import TypeVar, cast
from stripe._stripe_object import StripeObject

T = TypeVar("T", bound=StripeObject)


class UpdateableAPIResource(APIResource[T]):
    @classmethod
    def modify(cls, sid, **params) -> T:
        url = "%s/%s" % (cls.class_url(), quote_plus(sid))
        return cast(T, cls._static_request("post", url, params=params))

    @_util.deprecated(
        "The `save` method is deprecated and will be removed in a future major version of the library. Use the class method `modify` on the resource instead."
    )
    def save(self, idempotency_key=None):
        updated_params = self.serialize(None)
        if updated_params:
            updated_params["idempotency_key"] = idempotency_key
            self._request_and_refresh(
                "post",
                self.instance_url(),
                params=updated_params,
                usage=["save"],
            )
        else:
            _util.logger.debug("Trying to save already saved object %r", self)
        return self



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import Dict, List
from typing_extensions import Literal, NotRequired


class PayoutCreateParams(RequestOptions):
    amount: int
    """
    A positive integer in cents representing how much to payout.
    """
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    description: NotRequired[str]
    """
    An arbitrary string attached to the object. Often useful for displaying to users.
    """
    destination: NotRequired[str]
    """
    The ID of a bank account or a card to send the payout to. If you don't provide a destination, we use the default external account for the specified currency.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    metadata: NotRequired[Dict[str, str]]
    """
    Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    method: NotRequired[Literal["instant", "standard"]]
    """
    The method used to send this payout, which is `standard` or `instant`. We support `instant` for payouts to debit cards and bank accounts in certain countries. Learn more about [bank support for Instant Payouts](https://stripe.com/docs/payouts/instant-payouts-banks).
    """
    payout_method: NotRequired[str]
    """
    The ID of a v2 FinancialAccount to send funds to.
    """
    source_type: NotRequired[Literal["bank_account", "card", "fpx"]]
    """
    The balance type of your Stripe balance to draw this payout from. Balances for different payment sources are kept separately. You can find the amounts with the Balances API. One of `bank_account`, `card`, or `fpx`.
    """
    statement_descriptor: NotRequired[str]
    """
    A string that displays on the recipient's bank or card statement (up to 22 characters). A `statement_descriptor` that's longer than 22 characters return an error. Most banks truncate this information and display it inconsistently. Some banks might not display it at all.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing import List
from typing_extensions import Literal, NotRequired, TypedDict


class EventDestinationListParams(TypedDict):
    include: NotRequired[List[Literal["webhook_endpoint.url"]]]
    """
    Additional fields to include in the response. Currently supports `webhook_endpoint.url`.
    """
    limit: NotRequired[int]
    """
    The page size.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe.climate._order import Order
    from stripe.params.climate._order_cancel_params import OrderCancelParams
    from stripe.params.climate._order_create_params import OrderCreateParams
    from stripe.params.climate._order_list_params import OrderListParams
    from stripe.params.climate._order_retrieve_params import (
        OrderRetrieveParams,
    )
    from stripe.params.climate._order_update_params import OrderUpdateParams


class OrderService(StripeService):
    def list(
        self,
        params: Optional["OrderListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Order]":
        """
        Lists all Climate order objects. The orders are returned sorted by creation date, with the
        most recently created orders appearing first.
        """
        return cast(
            "ListObject[Order]",
            self._request(
                "get",
                "/v1/climate/orders",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["OrderListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Order]":
        """
        Lists all Climate order objects. The orders are returned sorted by creation date, with the
        most recently created orders appearing first.
        """
        return cast(
            "ListObject[Order]",
            await self._request_async(
                "get",
                "/v1/climate/orders",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "OrderCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Order":
        """
        Creates a Climate order object for a given Climate product. The order will be processed immediately
        after creation and payment will be deducted your Stripe balance.
        """
        return cast(
            "Order",
            self._request(
                "post",
                "/v1/climate/orders",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "OrderCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Order":
        """
        Creates a Climate order object for a given Climate product. The order will be processed immediately
        after creation and payment will be deducted your Stripe balance.
        """
        return cast(
            "Order",
            await self._request_async(
                "post",
                "/v1/climate/orders",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        order: str,
        params: Optional["OrderRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Order":
        """
        Retrieves the details of a Climate order object with the given ID.
        """
        return cast(
            "Order",
            self._request(
                "get",
                "/v1/climate/orders/{order}".format(order=sanitize_id(order)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        order: str,
        params: Optional["OrderRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Order":
        """
        Retrieves the details of a Climate order object with the given ID.
        """
        return cast(
            "Order",
            await self._request_async(
                "get",
                "/v1/climate/orders/{order}".format(order=sanitize_id(order)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        order: str,
        params: Optional["OrderUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Order":
        """
        Updates the specified order by setting the values of the parameters passed.
        """
        return cast(
            "Order",
            self._request(
                "post",
                "/v1/climate/orders/{order}".format(order=sanitize_id(order)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        order: str,
        params: Optional["OrderUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Order":
        """
        Updates the specified order by setting the values of the parameters passed.
        """
        return cast(
            "Order",
            await self._request_async(
                "post",
                "/v1/climate/orders/{order}".format(order=sanitize_id(order)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def cancel(
        self,
        order: str,
        params: Optional["OrderCancelParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Order":
        """
        Cancels a Climate order. You can cancel an order within 24 hours of creation. Stripe refunds the
        reservation amount_subtotal, but not the amount_fees for user-triggered cancellations. Frontier
        might cancel reservations if suppliers fail to deliver. If Frontier cancels the reservation, Stripe
        provides 90 days advance notice and refunds the amount_total.
        """
        return cast(
            "Order",
            self._request(
                "post",
                "/v1/climate/orders/{order}/cancel".format(
                    order=sanitize_id(order),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def cancel_async(
        self,
        order: str,
        params: Optional["OrderCancelParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Order":
        """
        Cancels a Climate order. You can cancel an order within 24 hours of creation. Stripe refunds the
        reservation amount_subtotal, but not the amount_fees for user-triggered cancellations. Frontier
        might cancel reservations if suppliers fail to deliver. If Frontier cancels the reservation, Stripe
        provides 90 days advance notice and refunds the amount_total.
        """
        return cast(
            "Order",
            await self._request_async(
                "post",
                "/v1/climate/orders/{order}/cancel".format(
                    order=sanitize_id(order),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )
